@@ -1,24 +1,30 @@
-/* Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
-
-   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPLv2 as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-   
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; version 2 of the License.
-   
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-   for more details.
-   
-   You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation, Inc.,
-   51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+// Copyright (c) 2006, 2024, Oracle and/or its affiliates.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// https://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,10 +32,10 @@
 #ifdef WIN32
     #include <windows.h>
 #else
-    #include <ltdl.h>
+    #include <dlfcn.h>
 #endif
 
-char *szSyntax =
+const char szSyntax[] =
 "\n" \
 "**********************************************\n" \
 "* dltest                                     *\n" \
@@ -98,11 +104,11 @@ int main( int argc, char *argv[] )
     int i;
     for (i = 1; i < argc; i++)
     {
-      if (strncmp(argv[1],"lib:",4) == 0)
+      if (strncmp(argv[i],"lib:",4) == 0)
       {
         hModule = dltest_dlopen(argv[i]+4);     /* Open a new module */
       }
-      else if (strncmp(argv[1],"sym:",4) == 0)
+      else if (strncmp(argv[i],"sym:",4) == 0)
       {
         dltest_dlsym(hModule,argv[i]+4);
       }
@@ -130,11 +136,13 @@ int main( int argc, char *argv[] )
 static void dltest_dlinit(void)
 {
 #ifndef WIN32
+/*
   if ( lt_dlinit() )
   {
     printf( "[%s][%d] ERROR: Failed to lt_dlinit()\n", __FILE__, __LINE__ );
     exit( 1 );
   }
+*/
 #endif
 }
 
@@ -150,17 +158,17 @@ static DLTestModule dltest_dlopen(const char *path)
                   GetLastError(),
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) &pszMsg,
-                  0, 
+                  0,
                   NULL);
-    printf("[%s][%d] ERROR LoadLibrary(): %s\n", __FILE__, __LINE__, pszMsg);
+    printf("[%s][%d] ERROR LoadLibrary(): %s\n", __FILE__, __LINE__, (char*)pszMsg);
     LocalFree(pszMsg);
     exit(1);
   }
 #else
-  DLTestModule hModule = lt_dlopen(path);
+  DLTestModule hModule = dlopen(path, RTLD_GLOBAL | RTLD_NOW);
   if ( !hModule )
   {
-    printf("[%s][%d] ERROR dlopen(): %s\n", __FILE__, __LINE__, lt_dlerror());
+    printf("[%s][%d] ERROR dlopen(): %s\n", __FILE__, __LINE__, dlerror());
     exit(1);
   }
 #endif /* WIN32 */
@@ -175,7 +183,7 @@ static void dltest_dlclose(DLTestModule hModule)
 #ifdef WIN32
   FreeLibrary(hModule);
 #else
-  lt_dlclose(hModule);
+  dlclose(hModule);
 #endif /* WIN32 */
 }
 
@@ -193,21 +201,21 @@ static void dltest_dlsym(DLTestModule hModule, const char *sym)
                   GetLastError(),
                   MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                   (LPTSTR) &pszMsg,
-                  0, 
+                  0,
                   NULL);
-    printf("[%s][%d] ERROR: Could not find %s. %s\n",__FILE__,__LINE__,sym,pszMsg);
+    printf("[%s][%d] ERROR: Could not find %s. %s\n",__FILE__,__LINE__,sym, (char*)pszMsg);
     LocalFree(pszMsg);
     FreeLibrary(hModule);
     exit(1);
   }
 #else
-  pFunc = (void (*)())lt_dlsym(hModule,sym);
-/* PAH - lt_dlerror() is not a good indicator of success    */
-/*		if ( (pError = lt_dlerror()) != NULL )              */
+  pFunc = (void (*)())dlsym(hModule,sym);
+/* PAH - dlerror() is not a good indicator of success    */
+/*		if ( (pError = dlerror()) != NULL )              */
   if ( !pFunc )
   {
     const char *pError;
-    if ( (pError = lt_dlerror()) != NULL )
+    if ( (pError = dlerror()) != NULL )
       printf("[%s][%d] ERROR: %s\n Could not find %s\n",__FILE__,__LINE__,pError,sym);
     else
       printf("[%s][%d] ERROR: Could not find %s\n",__FILE__,__LINE__,sym);
