@@ -1,21 +1,21 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of MySQL Connector/ODBC, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -86,13 +86,12 @@ SQLColAttributeImpl(SQLHSTMT hstmt, SQLUSMALLINT column,
   STMT *stmt= (STMT *)hstmt;
   SQLCHAR *value= NULL;
   SQLINTEGER len= SQL_NTS;
-  uint errors;
   SQLRETURN rc= MySQLColAttribute(hstmt, column, field, &value, num_attr);
 
   if (value)
   {
     SQLCHAR *old_value= value;
-    len= strlen((char *)value);
+    len = (SQLINTEGER)strlen((char *)value);
 
     /* We set the error only when the result is intented to be returned */
     if ((char_attr || num_attr) && len > char_attr_max - 1)
@@ -202,7 +201,6 @@ SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
   SQLCHAR *value= NULL;
   SQLINTEGER len= SQL_NTS;
   SQLSMALLINT free_value= 0;
-  uint errors;
 
   SQLRETURN rc;
 
@@ -220,7 +218,7 @@ SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
   if (value)
   {
     SQLCHAR *old_value= value;
-    len= strlen((char *)value);
+    len = (SQLINTEGER)strlen((char *)value);
 
     /* We set the error only when the result is intented to be returned */
     if (name && len > name_max - 1)
@@ -255,7 +253,7 @@ SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd, SQLCHAR *in, SQLSMALLINT in_len,
   CHECK_HANDLE(hdbc);
 
   if (in_len == SQL_NTS)
-    in_len= strlen((char *)in);
+    in_len = (SQLSMALLINT)strlen((char *)in);
   if (!out_len)
     out_len= &dummy_out;
 
@@ -291,12 +289,13 @@ SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd, SQLCHAR *in, SQLSMALLINT in_len,
   {
     uint errors;
     /* Now we have to convert SQLWCHAR back into a SQLCHAR. */
-    *out_len= (SQLSMALLINT)sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
-                                      outw, *out_len, &errors);
-    /* TODO what to do with errors? */
-
+    sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
+                            outw, *out_len, &errors);
     if (*out_len > out_max - 1)
-      rc= ((DBC*)hdbc)->set_error("01004", NULL, 0);
+    {
+      ((DBC*)hdbc)->set_error("01004", "String data, right truncated.", 0);
+      rc = SQL_SUCCESS_WITH_INFO;
+    }
   }
 
 error:
@@ -404,10 +403,7 @@ SQLGetConnectAttrImpl(SQLHDBC hdbc, SQLINTEGER attribute, SQLPOINTER value,
 
   if (char_value)
   {
-    SQLINTEGER len= SQL_NTS;
-    uint errors;
-
-    len= strlen((char *)char_value);
+    SQLINTEGER len = (SQLINTEGER)strlen((char *)char_value);
 
     /*
       This check is inside the statement, which does not
@@ -415,7 +411,7 @@ SQLGetConnectAttrImpl(SQLHDBC hdbc, SQLINTEGER attribute, SQLPOINTER value,
       see: "if (char_value)"
     */
     if (len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value && value_max > 1)
       strmake((char *)value, (char *)char_value, value_max - 1);
@@ -446,8 +442,6 @@ SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR *cursor, SQLSMALLINT cursor_max,
 {
   STMT *stmt= (STMT *)hstmt;
   SQLCHAR *name;
-  SQLINTEGER len;
-  uint errors;
 
   LOCK_STMT(stmt);
   CLEAR_STMT_ERROR(stmt);
@@ -456,7 +450,7 @@ SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR *cursor, SQLSMALLINT cursor_max,
     return stmt->set_error(MYERR_S1090, NULL, 0);
 
   name= MySQLGetCursorName(hstmt);
-  len= strlen((char *)name);
+  SQLINTEGER len = (SQLINTEGER)strlen((char *)name);
 
   if (cursor && cursor_max > 1)
     strmake((char *)cursor, (char *)name, cursor_max - 1);
@@ -480,7 +474,6 @@ SQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle,
 {
   DBC *dbc;
   SQLCHAR *value= NULL;
-  SQLINTEGER len= SQL_NTS;
 
   SQLRETURN rc= SQL_SUCCESS;
 
@@ -506,19 +499,17 @@ SQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle,
 
   if (value)
   {
-    uint errors;
-    len= strlen((char *)value);
+    size_t len = strlen((char *)value);
 
     /* We set the error only when the result is intented to be returned */
     if (info && len > info_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (info_len)
       *info_len= (SQLSMALLINT)len;
 
     if (info && info_max > 1)
       strmake((char *)info, (char *)value, info_max - 1);
-
   }
 
   return rc;
@@ -547,8 +538,6 @@ SQLGetDiagRecImpl(SQLSMALLINT handle_type, SQLHANDLE handle,
   SQLRETURN rc;
   DBC *dbc;
   SQLCHAR *msg_value= NULL, *sqlstate_value= NULL;
-  SQLINTEGER len= SQL_NTS;
-  uint errors;
 
   if (handle == NULL)
   {
@@ -583,14 +572,14 @@ SQLGetDiagRecImpl(SQLSMALLINT handle_type, SQLHANDLE handle,
 
   if (msg_value)
   {
-    len= strlen((char *)msg_value);
+    size_t len= strlen((char *)msg_value);
 
     /*
       We set the error only when the result is intented to be returned
       and message_max is greaater than 0
     */
     if (message && message_max && len > message_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (message_len)
       *message_len= (SQLSMALLINT)len;
@@ -615,32 +604,27 @@ SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT type, SQLPOINTER value,
 {
   DBC *dbc= (DBC *)hdbc;
   SQLCHAR *char_value= NULL;
-  SQLINTEGER len= SQL_NTS;
-  uint errors;
-
-  SQLRETURN rc;
 
   CHECK_HANDLE(hdbc);
 
-  rc= MySQLGetInfo(hdbc, type, &char_value, value, value_len);
+  SQLRETURN rc = MySQLGetInfo(hdbc, type, &char_value, value, value_len);
 
   if (char_value)
   {
-    len= strlen((char *)char_value);
+    size_t len = strlen((char *)char_value);
 
     /*
       MSSQL implementation does not return the truncation warning if the
       value is not NULL and value_max is 0
      */
     if (value && value_max && len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value && value_max > 1)
       strmake((char *)value, (char *)char_value, value_max - 1);
 
     if (value_len)
       *value_len= (SQLSMALLINT)len;
-
   }
 
   return rc;
@@ -677,7 +661,7 @@ SQLNativeSql(SQLHDBC hdbc, SQLCHAR *in, SQLINTEGER in_len,
 
   if (in_len == SQL_NTS)
   {
-    in_len= strlen((char *)in);
+    in_len = (SQLINTEGER)strlen((char *)in);
   }
 
   if (out_len)
@@ -687,7 +671,7 @@ SQLNativeSql(SQLHDBC hdbc, SQLCHAR *in, SQLINTEGER in_len,
 
   if (out && in_len >= out_max)
   {
-    rc= set_conn_error((DBC *)hdbc, MYERR_01004, NULL, 0);
+    rc = ((DBC *)hdbc)->set_error(MYERR_01004, NULL, 0);
   }
 
   if(out_max > 0)
@@ -725,7 +709,7 @@ SQLPrepareImpl(SQLHSTMT hstmt, SQLCHAR *str, SQLINTEGER str_len,
     we can pass it straight through. Otherwise it needs to be converted to
     the connection character set (probably utf-8).
   */
-  return MySQLPrepare(hstmt, str, str_len, false, false, force_prepare);
+  return MySQLPrepare(hstmt, str, str_len, false, force_prepare);
 }
 
 
@@ -786,6 +770,8 @@ SQLProcedures(SQLHSTMT hstmt,
 
   rc= MySQLProcedures(hstmt, catalog, catalog_len, schema, schema_len,
                       proc, proc_len);
+  // Remove parameters
+  ((STMT *)hstmt)->free_reset_params();
 
   return rc;
 }
@@ -984,8 +970,8 @@ SQLBrowseConnect(SQLHDBC hdbc, SQLCHAR *in, SQLSMALLINT in_len,
 {
   CHECK_HANDLE(hdbc);
 
-  return set_conn_error((DBC*)hdbc,MYERR_S1000,
-                        "Driver does not support this API", 0);
+  return ((DBC*)hdbc)->set_error(MYERR_S1000,
+    "Driver does not support this API", 0);
 }
 
 
