@@ -1,21 +1,21 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of MySQL Connector/ODBC, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -114,13 +114,13 @@ static const char *find_used_table(STMT *stmt)
 */
 char *check_if_positioned_cursor_exists(STMT *pStmt, STMT **pStmtCursor)
 {
-  const char * cursorName= get_cursor_name(&pStmt->query);
+  const char * cursorName = pStmt->query.get_cursor_name();
 
   if (cursorName != NULL)
   {
 
     DBC  *dbc= (DBC *)pStmt->dbc;
-    char * wherePos= get_token(&pStmt->query, TOKEN_COUNT(&pStmt->query)- 4);
+    const char *wherePos = pStmt->query.get_token((uint)pStmt->query.token_count() - 4);
 
     if (wherePos > GET_QUERY(&pStmt->query))
     {
@@ -224,7 +224,8 @@ static my_bool check_if_usable_unique_key_exists(STMT *stmt)
 
   /* Use SHOW KEYS FROM table to check for keys. */
   pos= myodbc_stpmov(buff, "SHOW KEYS FROM `");
-  pos+= mysql_real_escape_string(stmt->dbc->mysql, pos, table, strlen(table));
+  pos+= mysql_real_escape_string(stmt->dbc->mysql, pos, table,
+    (unsigned long)strlen(table));
   pos= myodbc_stpmov(pos, "`");
 
   MYLOG_QUERY(stmt, buff);
@@ -280,11 +281,9 @@ static my_bool check_if_usable_unique_key_exists(STMT *stmt)
   @purpose : positions the data cursor to appropriate row
 */
 
-void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
+bool set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
 {
   long       nrow, row_pos;
-  MYSQL_RES  *result= stmt->result;
-
 
   /*
     If irow exists, then position the current row to point
@@ -297,12 +296,17 @@ void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
   {
     if (ssps_used(stmt))
     {
-       data_seek(stmt, row_pos);
-       IGNORE_THROW(stmt->fetch_row());
+      data_seek(stmt, row_pos);
+      MYSQL_ROW r = nullptr;
+      IGNORE_THROW(r = stmt->fetch_row());
+      if (!r)
+        return false;
     }
     else
     {
       MYSQL_ROWS *dcursor;
+      MYSQL_RES  *result= stmt->result;
+
       dcursor= result->data->data;
 
       if (dcursor)
@@ -311,6 +315,8 @@ void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
         {
           dcursor= dcursor->next;
         }
+      } else {
+        return false;
       }
 
       result->data_cursor= dcursor;
@@ -318,6 +324,7 @@ void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
 
     stmt->cursor_row= row_pos;
   }
+  return true;
 }
 
 
@@ -414,7 +421,7 @@ static SQLRETURN copy_rowdata(STMT *stmt, DESCREC *aprec,
 {
     SQLRETURN rc;
     /* Negative length means either NULL or DEFAULT, so we need 7 chars. */
-    SQLUINTEGER length= (*aprec->octet_length_ptr > 0 ?
+    size_t length = (*aprec->octet_length_ptr > 0 ?
                          *aprec->octet_length_ptr + 1 : 7);
 
     if (stmt->extend_buffer(length) == NULL)
@@ -658,8 +665,13 @@ static SQLRETURN build_where_clause_std( STMT * pStmt,
                                      SQLUSMALLINT     irow )
 {
     /* set our cursor to irow - we call assuming irow is valid */
-    set_current_cursor_data( pStmt, irow );
-
+    if (!set_current_cursor_data( pStmt, irow ))
+    {
+      // In this case we don't have data to do update/delete
+      // because the resultset is empty.
+      pStmt->set_error(MYERR_01S03);
+      return SQL_NO_DATA;
+    }
     /* simply append WHERE to our statement */
     str.append(" WHERE ");
 
@@ -793,7 +805,7 @@ static SQLRETURN build_set_clause_std(STMT *stmt, SQLULEN irow,
                                              stmt->ard->bind_offset_ptr,
                                              stmt->ard->bind_type,
                                              bind_length(arrec->concise_type,
-                                                         arrec->octet_length),
+                                                         (ulong)arrec->octet_length),
                                              irow);
         aprec->octet_length= arrec->octet_length;
         if (length == SQL_NTS)
@@ -875,8 +887,9 @@ SQLRETURN my_pos_update_std( STMT *             pStmtCursor,
 
     pStmtTemp = (STMT *)hStmtTemp;
 
-    if (my_SQLPrepare(pStmtTemp, (SQLCHAR *)query.c_str(), query.size(),
-                      false, true, false) != SQL_SUCCESS)
+    if (my_SQLPrepare(pStmtTemp, (SQLCHAR *)query.c_str(),
+                      (SQLINTEGER)query.size(),
+                      true, false) != SQL_SUCCESS)
     {
         my_SQLFreeStmt( pStmtTemp, SQL_DROP );
         return pStmt->set_error("HY000", "my_SQLPrepare() failed.", 0 );
@@ -904,8 +917,9 @@ SQLRETURN my_pos_update_std( STMT *             pStmtCursor,
         statement that is a non-positioned update.
         To check: do we really need that?
       */
-      if (my_SQLPrepare(pStmt, (SQLCHAR *)query.c_str(), query.size(),
-                        false, true, false) != SQL_SUCCESS)
+      if (my_SQLPrepare(pStmt, (SQLCHAR *)query.c_str(),
+                        (SQLINTEGER)query.size(),
+                        true, false) != SQL_SUCCESS)
         return SQL_ERROR;
       pStmt->dae_type= DAE_NORMAL;
     }
@@ -924,12 +938,12 @@ SQLRETURN my_pos_update_std( STMT *             pStmtCursor,
 
 static SQLRETURN fetch_bookmark(STMT *stmt)
 {
-  SQLUINTEGER  rowset_pos,rowset_end;
+  size_t rowset_pos, rowset_end;
   SQLRETURN    nReturn= SQL_SUCCESS;
   DESCREC *arrec;
   SQLPOINTER TargetValuePtr= NULL;
   long curr_bookmark_index= 0;
-  long tmp_array_size= 0;
+  size_t tmp_array_size= 0;
 
   IS_BOOKMARK_VARIABLE(stmt);
   arrec= desc_get_rec(stmt->ard, -1, FALSE);
@@ -954,7 +968,8 @@ static SQLRETURN fetch_bookmark(STMT *stmt)
       TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
                                         stmt->ard->bind_offset_ptr,
                                         stmt->ard->bind_type,
-                                        arrec->octet_length, rowset_pos - 1);
+                                        (SQLINTEGER)arrec->octet_length,
+                                        rowset_pos - 1);
     }
 
     curr_bookmark_index= atol((const char*) TargetValuePtr);
@@ -970,7 +985,7 @@ static SQLRETURN fetch_bookmark(STMT *stmt)
   } while ( ++rowset_pos <= rowset_end );
 
   stmt->ard->array_size= tmp_array_size;
-  stmt->rows_found_in_set= rowset_pos - 1;
+  stmt->rows_found_in_set = (uint)rowset_pos - 1;
 
   return nReturn;
 }
@@ -983,10 +998,10 @@ static SQLRETURN fetch_bookmark(STMT *stmt)
 
 static SQLRETURN setpos_delete_bookmark_std(STMT *stmt, std::string &query)
 {
-  SQLUINTEGER  rowset_pos,rowset_end;
+  size_t rowset_pos,rowset_end;
   my_ulonglong affected_rows= 0;
   SQLRETURN    nReturn= SQL_SUCCESS;
-  ulong        query_length;
+  size_t        query_length;
   const char   *table_name;
   DESCREC *arrec;
   SQLPOINTER TargetValuePtr= NULL;
@@ -1003,7 +1018,7 @@ static SQLRETURN setpos_delete_bookmark_std(STMT *stmt, std::string &query)
 
   /* appened our table name to our DELETE statement */
   myodbc_append_quoted_name_std(query, table_name);
-  query_length= query.size();
+  query_length = query.size();
 
   IS_BOOKMARK_VARIABLE(stmt);
   arrec= desc_get_rec(stmt->ard, -1, FALSE);
@@ -1027,7 +1042,8 @@ static SQLRETURN setpos_delete_bookmark_std(STMT *stmt, std::string &query)
       TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
                                         stmt->ard->bind_offset_ptr,
                                         stmt->ard->bind_type,
-                                        arrec->octet_length, rowset_pos);
+                                        (SQLINTEGER)arrec->octet_length,
+                                        rowset_pos);
     }
 
     curr_bookmark_index= atol((const char*) TargetValuePtr);
@@ -1078,7 +1094,7 @@ static SQLRETURN setpos_delete_std(STMT *stmt, SQLUSMALLINT irow,
   SQLUINTEGER  rowset_pos,rowset_end;
   my_ulonglong affected_rows= 0;
   SQLRETURN    nReturn= SQL_SUCCESS;
-  ulong        query_length;
+  size_t       query_length;
   const char   *table_name;
 
   /* we want to work with base table name - we expect call to fail if more than one base table involved */
@@ -1144,10 +1160,10 @@ static SQLRETURN setpos_delete_std(STMT *stmt, SQLUSMALLINT irow,
 */
 static SQLRETURN setpos_update_bookmark_std(STMT *stmt, std::string &query)
 {
-  SQLUINTEGER  rowset_pos,rowset_end;
+  size_t rowset_pos,rowset_end;
   my_ulonglong affected= 0;
   SQLRETURN    nReturn= SQL_SUCCESS;
-  ulong        query_length;
+  size_t        query_length;
   const char   *table_name;
   DESCREC *arrec;
   SQLPOINTER TargetValuePtr= NULL;
@@ -1183,7 +1199,8 @@ static SQLRETURN setpos_update_bookmark_std(STMT *stmt, std::string &query)
       TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
                                         stmt->ard->bind_offset_ptr,
                                         stmt->ard->bind_type,
-                                        arrec->octet_length, rowset_pos);
+                                        (SQLINTEGER)arrec->octet_length,
+                                        rowset_pos);
     }
 
     curr_bookmark_index= atol((const char*) TargetValuePtr);
@@ -1238,7 +1255,7 @@ static SQLRETURN setpos_update_std(STMT *stmt, SQLUSMALLINT irow,
   SQLUINTEGER  rowset_pos,rowset_end;
   my_ulonglong affected= 0;
   SQLRETURN    nReturn= SQL_SUCCESS;
-  ulong        query_length;
+  size_t       query_length;
   const char   *table_name;
 
   if ( !(table_name= find_used_table(stmt)) )
@@ -1338,7 +1355,7 @@ static SQLRETURN batch_insert_std( STMT *stmt, SQLULEN irow, std::string &query
     SQLLEN       length;
     SQLUSMALLINT ncol;
     long i;
-    ulong        query_length= 0;           /* our original query len so we can reset pos if break_insert   */
+    size_t       query_length= 0;           /* our original query len so we can reset pos if break_insert   */
     my_bool      break_insert= FALSE;       /* true if we are to exceed max data size for transmission
                                                but this seems to be misused */
     DESCREC aprec_(DESC_PARAM, DESC_APP),
@@ -1420,7 +1437,7 @@ static SQLRETURN batch_insert_std( STMT *stmt, SQLULEN irow, std::string &query
                                                        stmt->ard->bind_offset_ptr,
                                                        stmt->ard->bind_type,
                                                        bind_length(arrec->concise_type,
-                                                                   arrec->octet_length),
+                                                                   (ulong)arrec->octet_length),
                                                        count);
                 }
 
@@ -1478,8 +1495,6 @@ static SQLRETURN batch_insert_std( STMT *stmt, SQLULEN irow, std::string &query
     if (stmt->stmt_options.bookmarks == SQL_UB_VARIABLE)
     {
       ulong copy_bytes= 0;
-      int _len= 0;
-      char _value[21];
       DESCREC *arrec;
       long max_row;
 
@@ -1503,7 +1518,7 @@ static SQLRETURN batch_insert_std( STMT *stmt, SQLULEN irow, std::string &query
             TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
                                               stmt->ard->bind_offset_ptr,
                                               stmt->ard->bind_type,
-                                              arrec->octet_length,
+                                              (SQLINTEGER)arrec->octet_length,
                                               i);
           }
 
@@ -1514,11 +1529,11 @@ static SQLRETURN batch_insert_std( STMT *stmt, SQLULEN irow, std::string &query
                                           stmt->ard->bind_type,
                                           sizeof(SQLLEN), i);
           }
-
-          _len= sprintf(_value, "%ld", i + 1);
-          res= sql_get_bookmark_data(stmt, arrec->concise_type, (uint)0,
+          std::string sval = std::to_string(i + 1);
+          res = sql_get_bookmark_data(stmt, arrec->concise_type, (uint)0,
                                 TargetValuePtr, arrec->octet_length,
-                                pcbValue, _value, _len, arrec);
+                                pcbValue, (char*)sval.data(),
+                                (ulong)sval.length(), arrec);
 
           if (!SQL_SUCCEEDED(res))
           {
@@ -1706,7 +1721,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 std::string del_query("DELETE FROM ");
                 del_query.reserve(1024);
 
-                ret = setpos_delete_std( stmt, irow, del_query );
+                ret = setpos_delete_std( stmt, (SQLUSMALLINT)irow, del_query );
                 break;
             }
 
@@ -1727,14 +1742,14 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 std::string upd_query("UPDATE ");
                 upd_query.reserve(1024);
 
-                ret= setpos_update_std(stmt, irow, upd_query);
+                ret = setpos_update_std(stmt, (SQLUSMALLINT)irow, upd_query);
                 break;
             }
 
         case SQL_ADD:
             {
                 const char  *   table_name;
-                SQLUSMALLINT    nCol        = 0;
+                unsigned int    nCol        = 0;
 
                 if (!stmt->dae_type && stmt->is_dynamic_cursor() &&
                     set_dynamic_result(stmt))
@@ -1816,7 +1831,7 @@ MySQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *name, SQLSMALLINT len)
     return stmt->set_error( MYERR_S1009, NULL, 0);
 
   if (len == SQL_NTS)
-    len= strlen((char *)name);
+    len = (SQLSMALLINT)strlen((char *)name);
 
   if (len < 0)
     return stmt->set_error( MYERR_S1009, NULL, 0);
