@@ -1,21 +1,21 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +40,7 @@
 #include "installer.h"
 
 /* Disable _attribute__ on non-gcc compilers. */
-#if !defined(__attribute__) && !defined(__GNUC__)
+#if !defined(__attribute__) && !defined(__GNUC__) && !defined(__clang__)
 # define __attribute__(arg)
 #endif
 
@@ -233,7 +233,7 @@ extern std::mutex global_fido_mutex;
 /* data-at-exec handling done for current SQLSetPos() call */
 #define DAE_SETPOS_DONE 10
 
-#define DONT_USE_LOCALE_CHECK(STMT) if (!STMT->dbc->ds->dont_use_set_locale)
+#define DONT_USE_LOCALE_CHECK(STMT) if (!STMT->dbc->ds.opt_NO_LOCALE)
 
 #if defined _WIN32
 
@@ -248,7 +248,7 @@ extern std::mutex global_fido_mutex;
 
   #define __LOCALE_RESTORE() \
     { \
-      setlocale(LC_NUMERIC, default_locale); \
+      setlocale(LC_NUMERIC, default_locale.c_str()); \
       _configthreadlocale(loc); \
     }
 
@@ -277,7 +277,7 @@ extern std::mutex global_fido_mutex;
 
   #define __LOCALE_RESTORE() \
       { \
-        setlocale(LC_NUMERIC, default_locale); \
+        setlocale(LC_NUMERIC, default_locale.c_str()); \
       }
 #endif
 
@@ -297,37 +297,9 @@ typedef struct {
   size_t offset; /* offset of field in struct */
 } desc_field;
 
-struct tempBuf
-{
-	char *buf;
-	size_t buf_len;
-  size_t cur_pos;
-
-	tempBuf(size_t size = 16384);
-
-  tempBuf(const tempBuf& b);
-
-  char* extend_buffer(char *to, size_t len);
-  char* extend_buffer(size_t len);
-
-  // Append data to the current buffer
-  char* add_to_buffer(const char *from, size_t len);
-
-  char* add_to_buffer(char *to, const char *from, size_t len);
-  void remove_trail_zeroes();
-  void reset();
-
-  operator bool();
-
-  void operator =(const tempBuf& b);
-
-	~tempBuf();
-};
-
 /* descriptor */
 struct STMT;
 
-
 struct DESCREC{
   /* ODBC spec fields */
   SQLINTEGER  auto_unique_value; /* row only */
@@ -397,7 +369,7 @@ struct DESCREC{
 
     void add_param_data(const char *chunk, unsigned long length);
 
-    SQLINTEGER val_length()
+    size_t val_length()
     {
       // Return the current position, not the buffer length
       return tempbuf.cur_pos;
@@ -620,7 +592,7 @@ struct DBC
                 *cxn_charset_info = nullptr;
   MY_SYNTAX_MARKERS *syntax = nullptr;
   // data source used to connect (parsed or stored)
-  DataSource    *ds = nullptr;
+  DataSource    ds;
   // value of the sql_select_limit currently set for a session
   //   (SQLULEN)(-1) if wasn't set
   SQLULEN       sql_select_limit = -1;
@@ -650,6 +622,10 @@ struct DBC
 
   void set_charset(std::string charset);
   SQLRETURN set_charset_options(const char* charset);
+  SQLRETURN set_error(myodbc_errid errid, const char* errtext,
+    SQLINTEGER errcode);
+  SQLRETURN execute_query(const char *query,
+    SQLULEN query_length, my_bool req_lock);
 };
 
 
@@ -663,7 +639,7 @@ struct MY_LIMIT_CLAUSE
 {
   unsigned long long  offset;
   unsigned int        row_count;
-  char                *begin, *end;
+  const char                *begin, *end;
   MY_LIMIT_CLAUSE(unsigned long long offs, unsigned int rc, char* b, char *e) :
     offset(offs), row_count(rc), begin(b), end(e)
   {}
@@ -672,16 +648,28 @@ struct MY_LIMIT_CLAUSE
 
 struct MY_LIMIT_SCROLLER
 {
-   char               *query, *offset_pos;
+   tempBuf buf;
+   char *query, *offset_pos;
    unsigned int       row_count;
    unsigned long long start_offset;
    unsigned long long next_offset, total_rows, query_len;
 
-   MY_LIMIT_SCROLLER() : query(NULL), offset_pos(NULL), row_count(0),
-                         start_offset(0), next_offset(0), total_rows(0),
-                         query_len(0)
-     {}
+   MY_LIMIT_SCROLLER() : buf(1024), query(buf.buf), offset_pos(query),
+                         row_count(0), start_offset(0), next_offset(0),
+                         total_rows(0), query_len(0)
+   {}
 
+   void extend_buf(size_t new_size) {
+     // Calculate the current offset
+     size_t offset = offset_pos - query;
+     buf.extend_buffer(new_size);
+
+     // Point to the new buffer and calculate the new offset position
+     query = buf.buf;
+     offset_pos = query + offset;
+   }
+
+   void reset() { next_offset = 0; offset_pos = query; }
 };
 
 /* Statement primary key handler for cursors */
@@ -722,13 +710,13 @@ enum OUT_PARAM_STATE
 
 #define CAT_SCHEMA_SET_FULL(STMT, C, S, V, CZ, SZ, CL, SL) { \
   bool cat_is_set = false; \
-  if (!STMT->dbc->ds->no_catalog && (CL || !SL)) \
+  if (!STMT->dbc->ds.opt_NO_CATALOG && (CL || !SL)) \
   { \
     C = V;\
     S = nullptr; \
     cat_is_set = true; \
   } \
-  if (!STMT->dbc->ds->no_schema && !cat_is_set && SZ) \
+  if (!STMT->dbc->ds.opt_NO_SCHEMA && !cat_is_set && SZ) \
   { \
     S = V; \
     C = nullptr; \
@@ -753,9 +741,7 @@ struct GETDATA{
 
   GETDATA() : column(0), source(NULL), latest_bytes(0),
               latest_used(0), src_offset(0), dst_bytes(0), dst_offset(0)
-  {
-    memchr(latest, 0, sizeof(latest));
-  }
+  {}
 };
 
 struct ODBC_RESULTSET
@@ -800,10 +786,7 @@ struct xstring : public std::string
 
   xstring(char* s) : m_is_null(s == nullptr),
                   Base(s == nullptr ? "" : std::forward<char*>(s))
-  {
-    if (m_is_null)
-      m_is_null = true;
-  }
+  {}
 
   template <class T>
   xstring(T &&s) : Base(std::forward<T>(s))
@@ -910,7 +893,7 @@ struct ROW_STORAGE
     {
       auto &data = m_data[m_cur_row * m_cnum + i];
       *(bind[i].is_null) = data.is_null();
-      *(bind[i].length) = data.is_null() ? -1 : data.length();
+      *(bind[i].length) = (unsigned long)(data.is_null() ? -1 : data.length());
       if (!data.is_null())
       {
         size_t copy_zero = bind[i].buffer_length > *(bind[i].length) ? 1 : 0;
@@ -953,22 +936,66 @@ struct ODBCEXCEPTION
 
 struct ODBC_STMT
 {
-  MYSQL_STMT *m_stmt;
+  MYSQL_STMT *m_stmt = nullptr;
 
-  ODBC_STMT(MYSQL *mysql) { m_stmt = mysql_stmt_init(mysql); }
+  ODBC_STMT(MYSQL *mysql) {
+    if (mysql)
+    m_stmt = mysql_stmt_init(mysql);
+  }
   operator MYSQL_STMT*() { return m_stmt; }
-  ~ODBC_STMT() { mysql_stmt_close(m_stmt); }
+  ~ODBC_STMT() {
+    if (m_stmt)
+    mysql_stmt_close(m_stmt);
+  }
 };
 
 
+class charPtrBuf {
+  private:
+
+  std::vector<char*> m_buf;
+  MYSQL_ROW m_external_val = nullptr;
+
+  public:
+
+  void reset() {
+     m_buf.clear();
+     m_external_val = nullptr;
+   }
+
+   void set_size(size_t size) {
+     m_buf.resize(size);
+     m_external_val = nullptr;
+   }
+
+  void set(const void* data, size_t size) {
+    set_size(size);
+    char *buf = (char*)data;
+    std::vector<char*> tmpVec(size, buf);
+    m_buf = tmpVec;
+  }
+
+  operator MYSQL_ROW() const {
+    if (m_external_val || m_buf.size())
+      return (MYSQL_ROW)(m_external_val ? m_external_val : m_buf.data());
+    return nullptr;
+  }
+
+  charPtrBuf &operator=(const MYSQL_ROW external_val) {
+    reset();
+    m_external_val = external_val;
+    return *this;
+  }
+};
+
 struct STMT
 {
   DBC               *dbc;
   MYSQL_RES         *result;
-  MEM_ROOT          alloc_root;
   my_bool           fake_result;
-  MYSQL_ROW	        array; // Holds row data directly from mysql resultset
-  MYSQL_ROW         result_array, current_values;
+  charPtrBuf        array;
+  charPtrBuf        result_array;
+  MYSQL_ROW         current_values;
   MYSQL_ROW         (*fix_fields)(STMT *stmt, MYSQL_ROW row);
   MYSQL_FIELD	      *fields;
   MYSQL_ROW_OFFSET  end_of_set;
@@ -979,6 +1006,7 @@ struct STMT
   MYERROR           error;
   STMT_OPTIONS      stmt_options;
   std::string       table_name;
+  std::string       catalog_name;
 
   MY_PARSED_QUERY	query, orig_query;
   std::vector<MYSQL_BIND> param_bind;
@@ -997,7 +1025,7 @@ struct STMT
 
   GETDATA           getdata;
 
-  uint		*order, order_count, param_count, current_param, rows_found_in_set;
+  uint		param_count, current_param, rows_found_in_set;
 
   enum MY_STATE state;
   enum MY_DUMMY_STATE dummy_state;
@@ -1072,12 +1100,12 @@ struct STMT
   */
   SQLRETURN set_error(const char *state);
 
-  STMT(DBC *d) : dbc(d), result(NULL), array(NULL), result_array(NULL),
+  STMT(DBC *d) : dbc(d), result(NULL), fake_result(false), array(), result_array(),
     current_values(NULL), fields(NULL), end_of_set(NULL),
     tempbuf(),
     stmt_options(dbc->stmt_options), lengths(nullptr), affected_rows(0),
     current_row(0), cursor_row(0), dae_type(0),
-    order(NULL), order_count(0), param_count(0), current_param(0),
+    param_count(0), current_param(0),
     rows_found_in_set(0),
     state(ST_UNKNOWN), dummy_state(ST_DUMMY_UNKNOWN),
     setpos_row(0), setpos_lock(0), setpos_op(0),
@@ -1093,17 +1121,15 @@ struct STMT
     ipd(&m_ipd),
     imp_ard(ard), imp_apd(apd)
   {
-    //list.data = this;
-    init_parsed_query(&query);
-    init_parsed_query(&orig_query);
     allocate_param_bind(10);
 
     LOCK_DBC(dbc);
     dbc->stmt_list.emplace_back(this);
-    //dbc->statements = list_add(dbc->statements, &list);
   }
 
   ~STMT();
+  void clear_query_attr_bind();
+  void reset_result_array();
 };
 
 namespace myodbc {
@@ -1147,9 +1173,9 @@ namespace myodbc {
     {
       SQLWSTRING  string_connect_in;
 
-      assert(params->driver && *params->driver);
-      ds_set_strattr(&params->name, NULL);
-      ds_to_kvpair(params, string_connect_in, ';');
+      assert((bool)params->opt_DRIVER);
+      params->opt_DSN.set_default(nullptr);
+      string_connect_in = params->to_kvpair(';');
       if (SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc) != SQL_SUCCESS)
       {
         throw MYERROR(SQL_HANDLE_ENV, henv, SQL_ERROR);
@@ -1200,8 +1226,7 @@ namespace myodbc {
   };
 };
 
-extern char *default_locale, *decimal_point, *thousands_sep;
-extern uint decimal_point_length,thousands_sep_length;
+extern std::string thousands_sep, decimal_point, default_locale;
 #ifndef _UNIX_
 extern HINSTANCE NEAR s_hModule;  /* DLL handle. */
 #endif
@@ -1227,9 +1252,7 @@ extern std::string default_plugin_location;
 # define MYSQL_TYPE_BIT 16
 #endif
 
-MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, char *query, char * query_end);
-void          delete_param_bind(DYNAMIC_ARRAY *param_bind);
-
+MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, const char *query, const char * query_end);
 
 #include "myutil.h"
 #include "stringutil.h"
@@ -1293,7 +1316,7 @@ SQLRETURN SQL_API MySQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute,
                                   SQLINTEGER *StringLengthPtr);
 SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType);
 SQLRETURN SQL_API MySQLPrepare(SQLHSTMT hstmt, SQLCHAR *query, SQLINTEGER len,
-                               bool dupe, bool reset_select_limit,
+                               bool reset_select_limit,
                                bool force_prepare);
 SQLRETURN SQL_API MySQLPrimaryKeys(SQLHSTMT hstmt,
                                    SQLCHAR *catalog, SQLSMALLINT catalog_len,
