@@ -1,21 +1,21 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of MySQL Connector/ODBC, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -198,7 +198,7 @@ SQLRETURN set_desc_error(DESC *        desc,
 
 void translate_error(char *save_state, myodbc_errid errid, uint mysql_err)
 {
-    char *state= myodbc3_errors[errid].sqlstate;
+    const char *state= myodbc3_errors[errid].sqlstate;
 
     switch (mysql_err)
     {
@@ -271,19 +271,6 @@ SQLRETURN set_env_error(ENV *env, myodbc_errid errid, const char *errtext,
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : sets the error information to connection handle.
-*/
-
-SQLRETURN set_conn_error(DBC *dbc, myodbc_errid errid, const char *errtext,
-                         SQLINTEGER errcode)
-{
-  dbc->error = MYERROR(errid, errtext, errcode, MYODBC_ERROR_PREFIX);
-  return dbc->error.retcode;
-}
-
-
 /*
   @type    : myodbc3 internal
   @purpose : sets a myodbc_malloc() failure on a MYSQL* connection
@@ -356,7 +343,7 @@ SQLRETURN set_handle_error(SQLSMALLINT HandleType, SQLHANDLE handle,
     case SQL_HANDLE_ENV:
       return set_env_error((ENV*)handle, errid, errtext, errcode);
     case SQL_HANDLE_DBC:
-      return set_conn_error((DBC*)handle, errid, errtext, errcode);
+      return ((DBC*)handle)->set_error(errid, errtext, errcode);
     case SQL_HANDLE_STMT:
     {
       STMT *stmt = (STMT*)handle;
@@ -426,7 +413,7 @@ MySQLGetDiagRec(SQLSMALLINT handle_type, SQLHANDLE handle, SQLSMALLINT record,
 
 bool is_odbc3_subclass(std::string sqlstate)
 {
-  char *states[]= { "01S00", "01S01", "01S02", "01S06", "01S07", "07S01",
+  const char *states[]= { "01S00", "01S01", "01S02", "01S06", "01S07", "07S01",
     "08S01", "21S01", "21S02", "25S01", "25S02", "25S03", "42S01", "42S02",
     "42S11", "42S12", "42S21", "42S22", "HY095", "HY097", "HY098", "HY099",
     "HY100", "HY101", "HY105", "HY107", "HY109", "HY110", "HY111", "HYT00",
@@ -549,16 +536,16 @@ MySQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle, SQLSMALLINT record,
       return SQL_ERROR;
 
     if (handle_type == SQL_HANDLE_DESC)
-      ds= desc->stmt->dbc->ds;
+      ds= &desc->stmt->dbc->ds;
     else if (handle_type == SQL_HANDLE_STMT)
-      ds= stmt->dbc->ds;
+      ds= &stmt->dbc->ds;
     else if (handle_type == SQL_HANDLE_DBC)
-      ds= dbc->ds;
+      ds= &dbc->ds;
     else
-      *char_value= (SQLCHAR *)"";
+      *char_value= (SQLCHAR*)"";
 
     if (ds)
-      *char_value= (SQLCHAR *)ds->name8;
+      *char_value = ds->opt_DSN;
     return SQL_SUCCESS;
   }
 
@@ -584,16 +571,16 @@ MySQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle, SQLSMALLINT record,
     if (record <= 0)
       return SQL_ERROR;
     if (handle_type == SQL_HANDLE_DESC)
-      ds= desc->stmt->dbc->ds;
+      ds = &desc->stmt->dbc->ds;
     else if (handle_type == SQL_HANDLE_STMT)
-      ds= stmt->dbc->ds;
+      ds = &stmt->dbc->ds;
     else if (handle_type == SQL_HANDLE_DBC)
-      ds= dbc->ds;
+      ds = &dbc->ds;
     else
       *char_value= (SQLCHAR *)"";
 
     if (ds)
-      *char_value= (SQLCHAR *)ds->server8;
+      *char_value= ds->opt_SERVER;
 
     return SQL_SUCCESS;
   }
