@@ -1,21 +1,21 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,13 +40,14 @@
   @purpose : internal function to execute query and return result
   frees query if query != stmt->query
 */
-SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
+SQLRETURN do_query(STMT *stmt, std::string query)
 {
     int error= SQL_ERROR, native_error= 0;
+    SQLULEN query_length = query.length();
     assert(stmt);
     LOCK_STMT_DEFER(stmt);
 
-    if (!query)
+    if (query.empty())
     {
       /* Probably error from insert_param */
       goto exit;
@@ -64,12 +65,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
       goto exit;
     }
 
-    if (query_length == 0)
-    {
-      query_length= strlen(query);
-    }
-
-    MYLOG_QUERY(stmt, query);
+    MYLOG_QUERY(stmt, query.c_str());
     DO_LOCK_STMT();
     if ( check_if_server_is_alive( stmt->dbc ) )
     {
@@ -86,24 +82,24 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
      * and when no musltiple statements is allowed - we can't now parse query
      * that well to detect multiple queries.
      */
-    if (stmt->dbc->ds->cursor_prefetch_number > 0
-        && !stmt->dbc->ds->allow_multiple_statements
+    if (stmt->dbc->ds.opt_PREFETCH > 0
+        && !stmt->dbc->ds.opt_MULTI_STATEMENTS
         && stmt->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY
-        && scrollable(stmt, query, query+query_length)
+        && scrollable(stmt, query.c_str(), query.c_str() + query_length)
         && !ssps_used(stmt))
     {
       /* we might want to read primary key info at this point, but then we have to
          know if we have a select from a single table...
        */
       ssps_close(stmt);
-      scroller_reset(stmt);
+      stmt->scroller.reset();
 
       stmt->scroller.row_count= calc_prefetch_number(
-                                      stmt->dbc->ds->cursor_prefetch_number,
+                                      stmt->dbc->ds.opt_PREFETCH,
                                       stmt->ard->array_size,
                                       stmt->stmt_options.max_rows);
 
-      scroller_create(stmt, query, query_length);
+      scroller_create(stmt, query.c_str(), query_length);
       scroller_move(stmt);
       MYLOG_QUERY(stmt, stmt->scroller.query);
 
@@ -111,7 +107,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
                                   (unsigned long)stmt->scroller.query_len);
     }
       /* Not using ssps for scroller so far. Relaxing a bit condition
-       if allow_multiple_statements option selected by primitive check if
+       if MULTI_STATEMENTS option selected by primitive check if
        this is a batch of queries */
     else if (ssps_used(stmt))
     {
@@ -150,7 +146,8 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
         goto exit;
       }
 
-      native_error= mysql_real_query(stmt->dbc->mysql,query,query_length);
+      native_error= mysql_real_query(stmt->dbc->mysql, query.c_str(),
+        (unsigned long)query_length);
     }
 
     MYLOG_QUERY(stmt, "query has been executed");
@@ -212,19 +209,14 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
 
 exit:
 
-    if (query != GET_QUERY(&stmt->query))
-    {
-      x_free(query);
-    }
-
     /*
       If the original query was modified, we reset stmt->query so that the
       next execution re-starts with the original query.
     */
     if (GET_QUERY(&stmt->orig_query))
     {
-      copy_parsed_query(&stmt->orig_query, &stmt->query);
-      reset_parsed_query(&stmt->orig_query, NULL, NULL, NULL);
+      stmt->query = stmt->orig_query;
+      stmt->orig_query.reset(NULL, NULL, NULL);
     }
 
     return error;
@@ -242,14 +234,12 @@ exit:
              so passing stmt->query->query can lead to memory leak.
 */
 
-SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
-                        SQLULEN *finalquery_length)
+SQLRETURN insert_params(STMT *stmt, SQLULEN row, std::string &finalquery)
 {
   assert(stmt);
-  char *query= GET_QUERY(&stmt->query);
+  const char *query= GET_QUERY(&stmt->query);
   uint i,length, had_info= 0;
   SQLRETURN rc= SQL_SUCCESS;
-  DECLARE_LOCALE_HANDLE
 
   LOCK_DBC(stmt->dbc);
 
@@ -259,7 +249,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
   {
     DESCREC *aprec= desc_get_rec(stmt->apd, i, FALSE);
     DESCREC *iprec= desc_get_rec(stmt->ipd, i, FALSE);
-    char *pos;
+    const char *pos;
     MYSQL_BIND * bind;
 
     if (stmt->dummy_state != ST_DUMMY_PREPARED &&
@@ -281,7 +271,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
     }
     else
     {
-      pos= get_param_pos(&stmt->query, i);
+      pos = stmt->query.get_param_pos(i);
       length= (uint) (pos-query);
 
       if (stmt->add_to_buffer(query, length) == NULL)
@@ -324,22 +314,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
       goto memerror;
     }
 
-    if (finalquery_length!= NULL)
-    {
-      *finalquery_length = stmt->buf_pos();
-    }
-
-    if (finalquery!=NULL)
-    {
-      char *dupquery = NULL;
-
-      if (!(dupquery = (char*)myodbc_memdup(stmt->buf(),
-        stmt->buf_pos(), MYF(0))))
-      {
-        goto memerror;
-      }
-      *finalquery= dupquery;
-    }
+    finalquery = std::string(stmt->buf(), stmt->buf_pos());
   }
 
   return rc;
@@ -395,7 +370,7 @@ BOOL allocate_param_buffer(MYSQL_BIND *bind, unsigned long length)
   }
   else if(bind->buffer_length < length)
   {
-    bind->buffer= myodbc_realloc(bind->buffer, length, MYF(0));
+    bind->buffer= myodbc_realloc(bind->buffer, length);
     bind->buffer_length= length;
   }
 
@@ -534,78 +509,78 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
     case SQL_C_BIT:
     case SQL_C_TINYINT:
     case SQL_C_STINYINT:
-        *length= my_int2str((long)*((signed char *)*res), buff, -10, 0) - buff;
+        *length = (long)(my_int2str((long)*((signed char *)*res), buff, -10, 0) - buff);
         *res= buff;
         break;
     case SQL_C_UTINYINT:
-        *length= my_int2str((long)*((unsigned char *)*res), buff, -10, 0) - buff;
+        *length = (long)(my_int2str((long)*((unsigned char *)*res), buff, -10, 0) - buff);
         *res= buff;
         break;
     case SQL_C_SHORT:
     case SQL_C_SSHORT:
-        *length= my_int2str((long)*((short int *)*res), buff, -10, 0) - buff;
+        *length = (long)(my_int2str((long)*((short int *)*res), buff, -10, 0) - buff);
         *res= buff;
         break;
     case SQL_C_USHORT:
-        *length= my_int2str((long)*((unsigned short int *)*res), buff, -10, 0) -
-                  buff;
+        *length = (long)(my_int2str((long)*((unsigned short int *)*res), buff, -10, 0) -
+                  buff);
         *res= buff;
         break;
     case SQL_C_LONG:
     case SQL_C_SLONG:
-        *length= my_int2str(*((SQLINTEGER*) *res), buff, -10, 0) - buff;
+        *length = (long)(my_int2str(*((SQLINTEGER*) *res), buff, -10, 0) - buff);
         *res= buff;
         break;
     case SQL_C_ULONG:
-        *length= my_int2str(*((SQLUINTEGER*) *res), buff, 10, 0) - buff;
+        *length = (long)(my_int2str(*((SQLUINTEGER*) *res), buff, 10, 0) - buff);
         *res= buff;
         break;
     case SQL_C_SBIGINT:
-        *length= myodbc_ll2str(*((longlong*) *res), buff, -10) - buff;
+        *length = (long)(myodbc_ll2str(*((longlong*) *res), buff, -10) - buff);
         *res= buff;
         break;
     case SQL_C_UBIGINT:
-        *length= myodbc_ll2str(*((ulonglong*) *res), buff, 10) - buff;
+        *length = (long)(myodbc_ll2str(*((ulonglong*) *res), buff, 10) - buff);
         *res= buff;
         break;
     case SQL_C_FLOAT:
       if ( iprec->concise_type != SQL_NUMERIC && iprec->concise_type != SQL_DECIMAL )
       {
-        sprintf(buff, "%.17e", *((float*) *res));
+        // Better precision
+        myodbc_d2str(*((float *)*res), buff, buff_max);
       }
       else
       {
-        /* We should perpare this data for string comparison */
-        sprintf(buff, "%.15e", *((float*) *res));
+        // We should perpare this data for string comparison, less precision
+        myodbc_d2str(*((float *)*res), buff, buff_max, false);
       }
-      delocalize_radix(buff);
-      *length= strlen(*res= buff);
+      *length = (long)strlen(*res= buff);
       break;
     case SQL_C_DOUBLE:
       if ( iprec->concise_type != SQL_NUMERIC && iprec->concise_type != SQL_DECIMAL )
       {
-        sprintf(buff,"%.17e",*((double*) *res));
+        myodbc_d2str(*((double *)*res), buff, buff_max);
       }
       else
       {
-        /* We should perpare this data for string comparison */
-        sprintf(buff,"%.15e",*((double*) *res));
+        // We should perpare this data for string comparison, less precision
+        myodbc_d2str(*((double*)*res), buff, buff_max, false);
       }
-      delocalize_radix(buff);
-      *length= strlen(*res= buff);
+      *length = (long)strlen(*res= buff);
       break;
     case SQL_C_DATE:
     case SQL_C_TYPE_DATE:
       {
         DATE_STRUCT *date= (DATE_STRUCT*) *res;
-        if (stmt->dbc->ds->min_date_to_zero && !date->year
+        if (stmt->dbc->ds.opt_MIN_DATE_TO_ZERO && !date->year
           && (date->month == date->day == 1))
         {
-          *length= sprintf(buff, "0000-00-00");
+          *length = myodbc_snprintf(buff, buff_max, "0000-00-00");
         }
         else
         {
-          *length= sprintf(buff, "%04d-%02d-%02d", date->year, date->month, date->day);
+          *length = myodbc_snprintf(buff, buff_max, "%04d-%02d-%02d",
+                                    date->year, date->month, date->day);
         }
         *res= buff;
         break;
@@ -620,8 +595,8 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
           return stmt->set_error("22008", "Not a valid time value supplied", 0);
         }
 
-        *length= sprintf(buff, "%02d:%02d:%02d",
-                                time->hour, time->minute, time->second);
+        *length = myodbc_snprintf(buff, buff_max, "%02d:%02d:%02d",
+                                  time->hour, time->minute, time->second);
         *res= buff;
         break;
       }
@@ -630,15 +605,16 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
       {
         TIMESTAMP_STRUCT *time= (TIMESTAMP_STRUCT*) *res;
 
-        if (stmt->dbc->ds->min_date_to_zero &&
+        if (stmt->dbc->ds.opt_MIN_DATE_TO_ZERO &&
             !time->year && (time->month == time->day == 1))
         {
-          *length= sprintf(buff, "0000-00-00 %02d:%02d:%02d", time->hour,
-                  time->minute, time->second);
+          *length = myodbc_snprintf(buff, buff_max, "0000-00-00 %02d:%02d:%02d",
+                                    time->hour, time->minute, time->second);
         }
         else
         {
-          *length= sprintf(buff, "%04d-%02d-%02d %02d:%02d:%02d",
+          *length = myodbc_snprintf(buff, buff_max,
+                    "%04d-%02d-%02d %02d:%02d:%02d",
                     time->year, time->month, time->day,
                     time->hour, time->minute, time->second);
         }
@@ -650,7 +626,8 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
           /* Start cleaning from the end */
           int tmp_pos= 9;
 
-          sprintf(tmp_buf, ".%09d", time->fraction);
+          myodbc_snprintf(tmp_buf, buff_max - *length,
+                          ".%09d", time->fraction);
 
           /*
             ODBC specification defines nanoseconds granularity for
@@ -681,7 +658,7 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
                       (SQLCHAR **) res,
                       (SQLCHAR) iprec->precision,
                       (SQLSCHAR) iprec->scale, &trunc);
-        *length= strlen(*res);
+        *length = (long)strlen(*res);
         /* TODO no way to return an error here? */
         if (trunc == SQLNUM_TRUNC_FRAC)
         {/* 01S07 SQL_SUCCESS_WITH_INFO */
@@ -705,28 +682,32 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
           /* Dirty-hackish */
           if (ssps_used(stmt))
           {
-            *length= sprintf(buff, "%d:%02d:00", interval->intval.day_second.hour,
-                                               interval->intval.day_second.minute);
+            *length = myodbc_snprintf(buff, buff_max, "%d:%02d:00",
+                                      interval->intval.day_second.hour,
+                                      interval->intval.day_second.minute);
           }
           else
           {
-            *length= sprintf(buff, "'%d:%02d:00'", interval->intval.day_second.hour,
-                                               interval->intval.day_second.minute);
+            *length = myodbc_snprintf(buff, buff_max, "'%d:%02d:00'",
+                                      interval->intval.day_second.hour,
+                                      interval->intval.day_second.minute);
           }
         }
         else
         {
           if (ssps_used(stmt))
           {
-            *length= sprintf(buff, "%d:%02d:%02d", interval->intval.day_second.hour,
-                                                 interval->intval.day_second.minute,
-                                                 interval->intval.day_second.second);
+            *length = myodbc_snprintf(buff, buff_max, "%d:%02d:%02d",
+                                      interval->intval.day_second.hour,
+                                      interval->intval.day_second.minute,
+                                      interval->intval.day_second.second);
           }
           else
           {
-            *length= sprintf(buff, "'%d:%02d:%02d'", interval->intval.day_second.hour,
-                                                 interval->intval.day_second.minute,
-                                                 interval->intval.day_second.second);
+            *length = myodbc_snprintf(buff, buff_max, "'%d:%02d:%02d'",
+                                      interval->intval.day_second.hour,
+                                      interval->intval.day_second.minute,
+                                      interval->intval.day_second.second);
           }
         }
 
@@ -805,7 +786,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
                                           apd->bind_offset_ptr,
                                           apd->bind_type,
                                           sizeof(SQLLEN), row);
-      length= *octet_length_ptr;
+      length = (long)*octet_length_ptr;
     }
 
     indicator_ptr= (SQLLEN*)ptr_offset_adjust(aprec->indicator_ptr,
@@ -816,7 +797,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
     if (aprec->data_ptr)
     {
       SQLINTEGER default_size= bind_length(aprec->concise_type,
-                                           aprec->octet_length);
+                                           (ulong)aprec->octet_length);
       data= (char*)ptr_offset_adjust(aprec->data_ptr, apd->bind_offset_ptr,
                               apd->bind_type, default_size, row);
     }
@@ -840,17 +821,17 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
       {
         if (aprec->concise_type == SQL_C_WCHAR)
         {
-          length= sqlwcharlen((SQLWCHAR *)data) * sizeof(SQLWCHAR);
+          length = (long)sqlwcharlen((SQLWCHAR *)data) * sizeof(SQLWCHAR);
         }
         else
         {
-          length= strlen(data);
+          length = (long)strlen(data);
         }
 
         if (!octet_length_ptr && aprec->octet_length > 0 &&
             aprec->octet_length != SQL_SETPARAM_VALUE_MAX)
         {
-          length= myodbc_min(length, aprec->octet_length);
+          length = (long)myodbc_min(length, aprec->octet_length);
         }
       }
       else
@@ -877,7 +858,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
     }
     else if (IS_DATA_AT_EXEC(octet_length_ptr))
     {
-        length= aprec->par.val_length();
+        length = (long)aprec->par.val_length();
         if ( !(data= aprec->par.val()) )
         {
           put_default_value(stmt, bind);
@@ -952,11 +933,6 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
         {
           TIMESTAMP_STRUCT ts;
 
-          /* For now I think it is safer to assume a dot is always a
-              separator */
-          /* aprec->concise_type == SQL_C_TYPE_TIMESTAMP
-          || aprec->concise_type == SQL_C_TIMESTAMP
-          || stmt->dbc->ds->dont_use_set_locale */
           str_to_ts(&ts, data, length, 1, TRUE);
 
           /* Overflow also possible if converted from other C types
@@ -970,7 +946,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
               Not sure if fraction should be considered as an overflow.
               In fact specs say about "time fields only"
             */
-          if (!stmt->dbc->ds->no_date_overflow && TIME_FIELDS_NONZERO(ts))
+          if (!stmt->dbc->ds.opt_NO_DATE_OVERFLOW && TIME_FIELDS_NONZERO(ts))
           {
             return stmt->set_error("22008", "Date overflow", 0);
           }
@@ -1009,17 +985,13 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
           {
             if (bind != NULL)
             {
-                if (bind_param(bind, data, length, MYSQL_TYPE_BLOB))
+              if (bind_param(bind, data, length, MYSQL_TYPE_BLOB))
               {
                 goto memerror;
               }
 
               goto out;
             }
-            else
-            {
-              stmt->add_to_buffer("_utf8mb4", 8);
-            }
           }
           else if (aprec->concise_type != SQL_C_WCHAR &&
                     dbc->cxn_charset_info->number !=
@@ -1034,12 +1006,6 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
 
               goto out;
             }
-            else
-            {
-              stmt->add_to_buffer("_", 1);
-              stmt->add_to_buffer(dbc->ansi_charset_info->csname,
-                                  strlen(dbc->ansi_charset_info->csname));
-            }
           }
           /* We have only added the introducer, data is added below. */
           break;
@@ -1082,11 +1048,13 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
 
           if (bind != NULL)
           {
-            length= sprintf(buff, "%02d:%02d:%02d", time->hour, time->minute, time->second);
+            length = myodbc_snprintf(buff, sizeof(buff), "%02d:%02d:%02d",
+                                     time->hour, time->minute, time->second);
           }
           else
           {
-            length= sprintf(buff, "'%02d:%02d:%02d'", time->hour, time->minute, time->second);
+            length = myodbc_snprintf(buff, sizeof(buff), "'%02d:%02d:%02d'",
+                                     time->hour, time->minute, time->second);
           }
 
           if (put_param_value(stmt, bind, buff, length))
@@ -1120,17 +1088,17 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
 
           if (bind != NULL)
           {
-            length= sprintf(buff, "%02d:%02d:%02d",
-                  hours,
-                  (int) time/100%100,
-                  (int) time%100);
+            length = myodbc_snprintf(buff, sizeof(buff), "%02d:%02d:%02d",
+                                     hours,
+                                     (int) time/100%100,
+                                     (int) time%100);
           }
           else
           {
-            length= sprintf(buff, "'%02d:%02d:%02d'",
-                  hours,
-                  (int) time/100%100,
-                  (int) time%100);
+            length = myodbc_snprintf(buff, sizeof(buff), "'%02d:%02d:%02d'",
+                                     hours,
+                                     (int) time/100%100,
+                                     (int) time%100);
           }
 
           if (put_param_value(stmt, bind, buff, length))
@@ -1163,12 +1131,12 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
         {
           char *to= buff, *from= data;
           char *end= from+length;
-          char *local_thousands_sep= thousands_sep;
-          char *local_decimal_point= decimal_point;
-          uint local_thousands_sep_length= thousands_sep_length;
-          uint local_decimal_point_length= decimal_point_length;
+          const char *local_thousands_sep = thousands_sep.c_str();
+          const char *local_decimal_point = decimal_point.c_str();
+          size_t local_thousands_sep_length = thousands_sep.length();
+          size_t local_decimal_point_length = decimal_point.length();
 
-          if (!stmt->dbc->ds->dont_use_set_locale)
+          if (!stmt->dbc->ds.opt_NO_LOCALE)
           {
             /* force use of . as decimal point */
             local_thousands_sep= ",";
@@ -1225,7 +1193,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
         SQLLEN transformed_len = 0;
         stmt->add_to_buffer(" 0x", 3);
         /* Make sure we have room for a fully-escaped string. */
-        if (!stmt->extend_buffer(length * 2))
+        if (!stmt->extend_buffer(length * 2 + 1))
         {
           goto memerror;
         }
@@ -1238,7 +1206,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
       {
         stmt->add_to_buffer("'", 1);
         /* Make sure we have room for a fully-escaped string. */
-        if (!(stmt->extend_buffer(length * 2)))
+        if (!(stmt->extend_buffer(length * 2 + 1)))
         {
           goto memerror;
         }
@@ -1274,7 +1242,7 @@ memerror:
 
 SQLRETURN do_my_pos_cursor_std( STMT *pStmt, STMT *pStmtCursor )
 {
-  char *          pszQuery= GET_QUERY(&pStmt->query);
+  const char *    pszQuery= GET_QUERY(&pStmt->query);
   std::string     query;
   SQLRETURN       nReturn;
 
@@ -1357,8 +1325,10 @@ BOOL map_error_to_param_status( SQLUSMALLINT *param_status_ptr, SQLRETURN rc)
 
 SQLRETURN my_SQLExecute( STMT *pStmt )
 {
-  char       *query, *cursor_pos;
-  int         dae_rec, is_select_stmt, one_of_params_not_succeded= 0;
+  std::string query;
+  char *cursor_pos;
+  int         dae_rec, one_of_params_not_succeded= 0;
+  bool is_select_stmt;
   int         connection_failure= 0;
   STMT       *pStmtCursor = pStmt;
   SQLRETURN   rc = 0;
@@ -1387,10 +1357,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
   if ((cursor_pos= check_if_positioned_cursor_exists(pStmt, &pStmtCursor)))
   {
     /* Save a copy of the query, because we're about to modify it. */
-    if (copy_parsed_query(&pStmt->query, &pStmt->orig_query))
-    {
-      return pStmt->set_error(MYERR_S1001,NULL,4001);
-    }
+    pStmt->orig_query = pStmt->query;
 
     /* Cursor statement use mysql_use_result - thus any operation
        will couse commands out of sync */
@@ -1409,7 +1376,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
 
   query= GET_QUERY(&pStmt->query);
 
-  is_select_stmt= is_select_statement(&pStmt->query);
+  is_select_stmt = pStmt->query.is_select_statement();
 
   /* if ssps is used for select query then convert it to non ssps
    single statement using UNION
@@ -1493,11 +1460,13 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
          query. */
       if (is_select_stmt && row < pStmt->apd->array_size - 1)
       {
-        rc= insert_params(pStmt, row, NULL, &length);
+        // Just ignore the dummy query
+        std::string dummy;
+        rc= insert_params(pStmt, row, dummy);
       }
       else
       {
-        rc= insert_params(pStmt, row, &query, &length);
+        rc= insert_params(pStmt, row, query);
       }
 
       /* Setting status for this paramset*/
@@ -1523,7 +1492,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
         if (row < pStmt->apd->array_size - 1)
         {
           const char * stmtsBinder= " UNION ALL ";
-          const ulong binderLength= strlen(stmtsBinder);
+          const size_t binderLength= strlen(stmtsBinder);
 
           pStmt->add_to_buffer(stmtsBinder, binderLength);
           length+= binderLength;
@@ -1535,23 +1504,18 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
     {
       if (!connection_failure)
       {
-        rc= do_query(pStmt, query, length);
+        rc = do_query(pStmt, query);
       }
       else
       {
-        if (query != GET_QUERY(&pStmt->query))
-        {
-          x_free(query);
-        }
-
         /*
           If the original query was modified, we reset stmt->query so that the
           next execution re-starts with the original query.
         */
         if (GET_QUERY(&pStmt->orig_query))
         {
-          copy_parsed_query(&pStmt->orig_query, &pStmt->query);
-          reset_parsed_query(&pStmt->orig_query, NULL, NULL, NULL);
+          pStmt->query = pStmt->orig_query;
+          pStmt->orig_query.reset(NULL, NULL, NULL);
         }
 
         /* with broken connection we always return error for all next queries */
@@ -1638,7 +1602,7 @@ static SQLRETURN select_dae_param_desc(STMT *stmt, DESC **apd, unsigned int *par
     case DAE_SETPOS_INSERT:
     case DAE_SETPOS_UPDATE:
       *apd= stmt->setpos_apd.get();
-      *param_count= stmt->ard->rcount();
+      *param_count = (unsigned int)stmt->ard->rcount();
       break;
     default:
       return stmt->set_error("HY010",
@@ -1672,7 +1636,7 @@ static SQLRETURN find_next_dae_param(STMT *stmt,  SQLPOINTER *token)
     if (IS_DATA_AT_EXEC(octet_length_ptr))
     {
       SQLINTEGER default_size= bind_length(aprec->concise_type,
-                                            aprec->octet_length);
+                                          (ulong)aprec->octet_length);
       stmt->current_param= i + 1;
       if (token)
       {
@@ -1695,16 +1659,15 @@ static SQLRETURN find_next_dae_param(STMT *stmt,  SQLPOINTER *token)
 static SQLRETURN execute_dae(STMT *stmt)
 {
   SQLRETURN rc;
-  char *query;
-  SQLULEN query_len = 0;
+  std::string query;
 
   switch (stmt->dae_type)
   {
   case DAE_NORMAL:
-    query= GET_QUERY(&stmt->query);
-    if (!SQL_SUCCEEDED(rc= insert_params(stmt, 0, &query, &query_len)))
+    query = GET_QUERY(&stmt->query);
+    if (!SQL_SUCCEEDED(rc= insert_params(stmt, 0, query)))
       break;
-    rc= do_query(stmt, query, query_len);
+    rc= do_query(stmt, query);
     break;
   case DAE_SETPOS_INSERT:
     stmt->dae_type= DAE_SETPOS_DONE;
@@ -1738,7 +1701,7 @@ static SQLRETURN find_next_out_stream(STMT *stmt, SQLPOINTER *token)
     if (token)
     {
       SQLINTEGER default_size= bind_length(rec->concise_type,
-                                          rec->octet_length);
+                                          (ulong)rec->octet_length);
       *token= ptr_offset_adjust(rec->data_ptr,
                                     stmt->ipd->bind_offset_ptr,
                                     stmt->ipd->bind_type,
@@ -1919,13 +1882,22 @@ SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
     interfere with the existing one. Therefore, locking is not needed in
     the following block.
   */
-  second= mysql_init(second);
+  second = new_mysql();
+
+  if (!second)
+  {
+    /*
+      TODO: Check that ODBC spec indeed require no SQLSTATE.
+      We do not set the SQLSTATE here, per the ODBC spec.
+    */
+    return SQL_ERROR;
+  }
 
   /** @todo need to preserve and use ssl params */
 
-  if (!mysql_real_connect(second, (const char*)dbc->ds->server8, (const char*)dbc->ds->uid8,
-                          (const char*)dbc->ds->pwd8, NULL, dbc->ds->port,
-                          (const char*)dbc->ds->socket8, 0))
+  if (!mysql_real_connect(second, dbc->ds.opt_SERVER, dbc->ds.opt_UID,
+                          dbc->ds.opt_PWD, NULL, dbc->ds.opt_PORT,
+                          dbc->ds.opt_SOCKET, 0))
   {
     /* We do not set the SQLSTATE here, per the ODBC spec. */
     return SQL_ERROR;
@@ -1934,8 +1906,8 @@ SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
   {
     char buff[40];
     /* buff is always big enough because max length of %lu is 15 */
-    sprintf(buff, "KILL /*!50000 QUERY */ %lu", mysql_thread_id(dbc->mysql));
-    if (mysql_real_query(second, buff, strlen(buff)))
+    myodbc_snprintf(buff, sizeof(buff), "KILL /*!50000 QUERY */ %lu", mysql_thread_id(dbc->mysql));
+    if (mysql_real_query(second, buff, (unsigned long)strlen(buff)))
     {
       mysql_close(second);
       /* We do not set the SQLSTATE here, per the ODBC spec. */
