@@ -1,21 +1,21 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -113,9 +113,6 @@ DBC::~DBC()
   if (env)
     env->remove_dbc(this);
 
-  if (ds)
-    ds_delete(ds);
-
   free_explicit_descriptors();
 }
 
@@ -135,6 +132,14 @@ SQLRETURN DBC::set_error(char * state)
 }
 
 
+SQLRETURN DBC::set_error(myodbc_errid errid, const char* errtext,
+  SQLINTEGER errcode)
+{
+  error = MYERROR(errid, errtext, errcode, MYODBC_ERROR_PREFIX);
+  return error.retcode;
+}
+
+
 /*
   @type    : myodbc3 internal
   @purpose : to allocate the environment handle and to maintain
@@ -239,7 +244,11 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
     if (mysql_get_client_version() < MIN_MYSQL_VERSION)
     {
         char buff[255];
-        sprintf(buff, "Wrong libmysqlclient library version: %ld.  MyODBC needs at least version: %ld", mysql_get_client_version(), MIN_MYSQL_VERSION);
+        myodbc_snprintf(buff, sizeof(buff),
+          "Wrong libmysqlclient library version: %ld. "
+          "MyODBC needs at least version: %ld",
+          mysql_get_client_version(), MIN_MYSQL_VERSION);
+
         return(set_env_error((ENV*)henv, MYERR_S1000, buff, 0));
     }
 
@@ -295,40 +304,37 @@ int reset_connection(DBC *dbc)
 
 int wakeup_connection(DBC *dbc)
 {
-  DataSource *ds= dbc->ds;
+  DataSource &ds = dbc->ds;
 
 #if MFA_ENABLED
-  if(ds->pwd1 && ds->pwd1[0])
+  if(ds.opt_PWD1)
   {
-    ds_get_utf8attr(ds->pwd1, &ds->pwd18);
     int fator = 2;
     mysql_options4(dbc->mysql, MYSQL_OPT_USER_PASSWORD,
-                   &fator,
-                   ds->pwd18);
+                   &fator, (const char*)ds.opt_PWD1);
   }
 
-  if(ds->pwd2 && ds->pwd2[0])
+  if (ds.opt_PWD2)
   {
     ds_get_utf8attr(ds->pwd2, &ds->pwd28);
     int fator = 2;
     mysql_options4(dbc->mysql, MYSQL_OPT_USER_PASSWORD,
                    &fator,
-                   ds->pwd28);
+                   (const char *)ds.opt_PWD2);
   }
 
-  if(ds->pwd3 && ds->pwd3[0])
+  if (ds.opt_PWD3)
   {
     ds_get_utf8attr(ds->pwd3, &ds->pwd38);
     int fator = 3;
     mysql_options4(dbc->mysql, MYSQL_OPT_USER_PASSWORD,
                    &fator,
-                   ds->pwd38);
+                   (const char *)ds.opt_PWD3);
   }
 #endif
 
-  if (mysql_change_user(dbc->mysql, ds_get_utf8attr(ds->uid, &ds->uid8),
-                                     ds_get_utf8attr(ds->pwd, &ds->pwd8),
-                                     ds_get_utf8attr(ds->database, &ds->database8)))
+  if (mysql_change_user(dbc->mysql, ds.opt_UID,
+      ds.opt_PWD, ds.opt_DATABASE))
   {
     return 1;
   }
@@ -377,7 +383,7 @@ SQLRETURN SQL_API SQLFreeConnect(SQLHDBC hdbc)
  */
 void STMT::allocate_param_bind(uint elements)
 {
-  if (dbc->ds->no_ssps)
+  if (dbc->ds.opt_NO_SSPS)
     return;
 
   if (param_bind.capacity() < elements)
@@ -404,11 +410,8 @@ int adjust_param_bind_array(STMT *stmt)
   @type    : myodbc3 internal
   @purpose : allocates the statement handle
 */
-SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
+SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc, SQLHSTMT *phstmt)
 {
-#ifndef _UNIX_
-  HGLOBAL  hstmt;
-#endif
   std::unique_ptr<STMT> stmt;
   DBC   *dbc= (DBC*) hdbc;
 
@@ -418,7 +421,7 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
 
   try
   {
-    stmt.reset (new STMT(dbc));
+    stmt.reset(new STMT(dbc));
   }
   catch (...)
   {
@@ -511,11 +514,9 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT f_option,
     stmt->free_fake_result((bool)(f_extra & FREE_STMT_CLEAR_RESULT));
 
     x_free(stmt->fields);   // TODO: Looks like STMT::fields is not used anywhere
-    x_free(stmt->result_array);
     stmt->result= 0;
     stmt->fake_result= 0;
     stmt->fields= 0;
-    stmt->result_array= 0;
     stmt->free_lengths();
     stmt->current_values= 0;   /* For SQLGetData */
     stmt->fix_fields= 0;
@@ -528,8 +529,7 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT f_option,
     if (f_option == FREE_STMT_RESET_BUFFERS)
     {
       free_result_bind(stmt);
-      x_free(stmt->array);
-      stmt->array= 0;
+      stmt->array.reset();
 
       return SQL_SUCCESS;
     }
@@ -552,8 +552,8 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT f_option,
 
     if (f_extra & FREE_STMT_CLEAR_RESULT)
     {
-      x_free(stmt->array);
-      stmt->array= 0;
+      stmt->array.reset();
+
       ssps_close(stmt);
       if (stmt->ssps != NULL)
       {
@@ -562,8 +562,8 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT f_option,
     }
 
     /* At this point, only FREE_STMT_RESET and SQL_DROP left out */
-    reset_parsed_query(&stmt->orig_query, NULL, NULL, NULL);
-    reset_parsed_query(&stmt->query, NULL, NULL, NULL);
+    stmt->orig_query.reset(NULL, NULL, NULL);
+    stmt->query.reset(NULL, NULL, NULL);
 
     stmt->param_count= 0;
 
@@ -691,7 +691,7 @@ SQLRETURN SQL_API SQLAllocHandle(SQLSMALLINT HandleType,
             break;
 
         default:
-            return set_conn_error((DBC*)InputHandle,MYERR_S1C00,NULL,0);
+            return ((DBC*)InputHandle)->set_error(MYERR_S1C00, NULL, 0);
     }
 
     return error;
