@@ -1,21 +1,21 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,7 +44,7 @@
 */
 
 #define if_forward_cache(st) ((st)->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY && \
-           (st)->dbc->ds->dont_cache_result)
+           (st)->dbc->ds.opt_NO_CACHE)
 #define is_connected(dbc)    ((dbc)->mysql && (dbc)->mysql->net.vio)
 #define trans_supported(db) ((db)->mysql->server_capabilities & CLIENT_TRANSACTIONS)
 #define autocommit_on(db) ((db)->mysql->server_status & SERVER_STATUS_AUTOCOMMIT)
@@ -52,10 +52,10 @@
 #define reset_ptr(x) {if (x) x= 0;}
 #define digit(A) ((int) (A - '0'))
 
-#define MYLOG_QUERY(A,B) {if ((A)->dbc->ds->save_queries) \
+#define MYLOG_QUERY(A,B) {if ((A)->dbc->ds.opt_LOG_QUERY) \
                query_print((A)->dbc->query_log,(char*) B);}
 
-#define MYLOG_DBC_QUERY(A,B) {if((A)->ds->save_queries) \
+#define MYLOG_DBC_QUERY(A,B) {if((A)->ds.opt_LOG_QUERY) \
                query_print((A)->query_log,(char*) B);}
 
 /* A few character sets we care about. */
@@ -84,48 +84,29 @@ typedef unsigned char * DYNAMIC_ELEMENT;
 typedef char * DYNAMIC_ELEMENT;
 #endif
 
-#if MYSQL_VERSION_ID >= 50100
 /* Same us MYODBC_FIELD_STRING(name, NAME_LEN, flags) */
 # define MYODBC_FIELD_NAME(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, NAME_LEN, 0, 0, 0, 0, 0, 0, \
+  {(char*)(name), (char*)(name), NullS, NullS, NullS, NullS, NullS, NAME_LEN, 0, 0, 0, 0, 0, 0, \
     0, 0, (flags), 0, UTF8_CHARSET_NUMBER, MYSQL_TYPE_VAR_STRING, NULL}
 # define MYODBC_FIELD_STRING(name, len, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, (len*SYSTEM_CHARSET_MBMAXLEN), 0, 0, 0, 0, 0, 0, \
+  {(char*)(name), (char*)(name), NullS, NullS, NullS, NullS, NullS, (len*SYSTEM_CHARSET_MBMAXLEN), 0, 0, 0, 0, 0, 0, \
     0, 0, (flags), 0, UTF8_CHARSET_NUMBER, MYSQL_TYPE_VAR_STRING, NULL}
 # define MYODBC_FIELD_SHORT(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 5, 5, 0, 0, 0, 0, 0, 0, \
+  {(char*)(name), (char*)(name), NullS, NullS, NullS, NullS, NullS, 5, 5, 0, 0, 0, 0, 0, 0, \
     0, (flags), 0, 0, MYSQL_TYPE_SHORT, NULL}
 # define MYODBC_FIELD_LONG(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 11, 11, 0, 0, 0, 0, 0, \
+  {(char*)(name), (char*)(name), NullS, NullS, NullS, NullS, NullS, 11, 11, 0, 0, 0, 0, 0, \
     0, 0, (flags), 0, 0, MYSQL_TYPE_LONG, NULL}
 # define MYODBC_FIELD_LONGLONG(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 20, 20, 0, 0, 0, 0, 0, \
+  {(char*)(name), (char*)(name), NullS, NullS, NullS, NullS, NullS, 20, 20, 0, 0, 0, 0, 0, \
     0, 0, (flags), 0, 0, MYSQL_TYPE_LONGLONG, NULL}
-#elif MYSQL_VERSION_ID >= 40100
-# define MYODBC_FIELD_NAME(name, flags) \
-{(name), (name), NullS, NullS, NullS, NullS, NullS, NAME_LEN*3, 0, 0, 0, 0, 0, 0, \
-  0, 0, (flags), 0, UTF8_CHARSET_NUMBER, MYSQL_TYPE_VAR_STRING, NULL}
-/* we use 3 here as SYSTEM_CHARSET_MBMAXLEN is not defined in v5.0 mysql_com.h */
-# define MYODBC_FIELD_STRING(name, len, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, (len) * 3, 0, 0, 0, 0, \
-    0, 0, 0, 0, (flags), 0, UTF8_CHARSET_NUMBER, MYSQL_TYPE_VAR_STRING}
-# define MYODBC_FIELD_SHORT(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 5, 5, 0, 0, 0, 0, 0, 0, \
-    0, (flags), 0, 0, MYSQL_TYPE_SHORT}
-# define MYODBC_FIELD_LONG(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 11, 11, 0, 0, 0, 0, 0, \
-    0, 0, (flags), 0, 0, MYSQL_TYPE_LONG}
-# define MYODBC_FIELD_LONGLONG(name, flags) \
-  {(name), (name), NullS, NullS, NullS, NullS, NullS, 20, 20, 0, 0, 0, 0, 0, \
-    0, 0, (flags), 0, 0, MYSQL_TYPE_LONGLONG }
-#endif
 
 /*
   Utility function prototypes that share among files
 */
 
 SQLRETURN         my_SQLPrepare (SQLHSTMT hstmt, SQLCHAR *szSqlStr,
-                                 SQLINTEGER cbSqlStr, bool dupe,
+                                 SQLINTEGER cbSqlStr,
                                  bool reset_select_limit,
                                  bool force_prepare);
 SQLRETURN         my_SQLExecute         (STMT * stmt);
@@ -133,11 +114,8 @@ SQLRETURN SQL_API my_SQLFreeStmt        (SQLHSTMT hstmt,SQLUSMALLINT fOption);
 SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT fOption,
                                          SQLUSMALLINT fExtra);
 SQLRETURN SQL_API my_SQLAllocStmt       (SQLHDBC hdbc,SQLHSTMT *phstmt);
-SQLRETURN         do_query              (STMT *stmt,char *query, SQLULEN query_length);
-SQLRETURN         insert_params         (STMT *stmt, SQLULEN row, char **finalquery,
-                                        SQLULEN *length);
-SQLRETURN odbc_stmt(DBC *dbc, const char *query, SQLULEN query_length,
-                    my_bool reqLock);
+SQLRETURN         do_query              (STMT *stmt, std::string query);
+SQLRETURN         insert_params         (STMT *stmt, SQLULEN row, std::string &finalquery);
 void      myodbc_link_fields (STMT *stmt,MYSQL_FIELD *fields,uint field_count);
 void      fix_row_lengths   (STMT *stmt, const long* fix_rules, uint row, uint field_count);
 void      fix_result_types  (STMT *stmt);
@@ -152,8 +130,6 @@ char *    check_if_positioned_cursor_exists (STMT *stmt, STMT **stmtNew);
 SQLRETURN insert_param  (STMT *stmt, MYSQL_BIND *bind, DESC *apd,
                         DESCREC *aprec, DESCREC *iprec, SQLULEN row);
 
-void reset_getdata_position   (STMT *stmt);
-
 SQLRETURN set_sql_select_limit(DBC *dbc, SQLULEN new_value, my_bool reqLock);
 SQLRETURN exec_stmt_query(STMT *stmt, const char *query, SQLULEN query_length,
                            my_bool reqLock);
@@ -268,13 +244,11 @@ void  myodbc_ov_init            (SQLINTEGER odbc_version);
 void  myodbc_sqlstate2_init     (void);
 void  myodbc_sqlstate3_init     (void);
 int   check_if_server_is_alive  (DBC *dbc);
-
+MYSQL *new_mysql                (void);
 bool   myodbc_append_quoted_name_std(std::string &str, const char *name);
 
 SQLRETURN set_handle_error          (SQLSMALLINT HandleType, SQLHANDLE handle,
                                     myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
-SQLRETURN set_conn_error(DBC *dbc,myodbc_errid errid, const char *errtext,
-                        SQLINTEGER errcode);
 SQLRETURN set_env_error (ENV * env,myodbc_errid errid, const char *errtext,
                         SQLINTEGER errcode);
 SQLRETURN copy_str_data (SQLSMALLINT HandleType, SQLHANDLE Handle,
@@ -287,7 +261,7 @@ SQLRETURN SQL_API my_SQLFreeEnv       (SQLHENV henv);
 
 void myodbc_end();
 my_bool set_dynamic_result        (STMT *stmt);
-void    set_current_cursor_data   (STMT *stmt,SQLUINTEGER irow);
+bool    set_current_cursor_data   (STMT *stmt,SQLUINTEGER irow);
 my_bool is_minimum_version        (const char *server_version,const char *version);
 int     myodbc_strcasecmp         (const char *s, const char *t);
 int     myodbc_casecmp            (const char *s, const char *t, uint len);
@@ -321,8 +295,6 @@ void *ptr_offset_adjust   (void *ptr, SQLULEN *bind_offset,
                           SQLINTEGER bind_type, SQLINTEGER default_size,
                           SQLULEN row);
 
-void free_internal_result_buffers(STMT *stmt);
-
 /* Functions used when debugging */
 void query_print          (FILE *log_file,char *query);
 FILE *init_query_log      (void);
@@ -353,7 +325,6 @@ const char *get_fractional_part   (const char * str, int len,
                                   SQLUINTEGER * fraction);
 /* Convert MySQL timestamp to full ANSI timestamp format. */
 char *          complete_timestamp  (const char * value, ulong length, char buff[21]);
-long double     myodbc_strtold             (const char *nptr, char **endptr);
 BOOL            myodbc_isspace      (CHARSET_INFO* cs, const char * begin, const char *end);
 BOOL            myodbc_isnum        (CHARSET_INFO* cs, const char * begin, const char *end);
 
@@ -414,7 +385,7 @@ unsigned long long get_uint64(STMT * stmt, ulong column_number, char* value,
                           ulong length);
 char *        get_string  (STMT *stmt, ulong column_number, char *value,
                           ulong *length, char * buffer);
-long double   get_double  (STMT *stmt, ulong column_number, char *value,
+double        get_double  (STMT *stmt, ulong column_number, char *value,
                           ulong length);
 BOOL          is_null     (STMT *stmt, ulong column_number, char *value);
 SQLRETURN     prepare     (STMT *stmt, char * query, SQLINTEGER query_length,
@@ -439,16 +410,16 @@ void stmt_result_free(STMT * stmt)
 
 
 /* scroller-related functions */
-void          scroller_reset      (STMT *stmt);
 unsigned int  calc_prefetch_number(unsigned int selected, SQLULEN app_fetchs,
                                    SQLULEN max_rows);
 BOOL          scroller_exists     (STMT * stmt);
-void          scroller_create     (STMT * stmt, char *query, SQLULEN len);
+void          scroller_create     (STMT * stmt, const char *query, SQLULEN len);
 
 unsigned long long  scroller_move (STMT * stmt);
 
 SQLRETURN     scroller_prefetch   (STMT * stmt);
-BOOL          scrollable          (STMT * stmt, char * query, char * query_end);
+bool          scrollable          (STMT * stmt, const char * query,
+                                  const char * query_end);
 
 /* my_prepared_stmt.c */
 void        ssps_init             (STMT *stmt);
@@ -462,7 +433,7 @@ BOOL        ssps_buffers_need_extending(STMT *stmt);
 
 template <typename T>
 T ssps_get_int64 (STMT *stmt, ulong column_number, char *value, ulong length);
-long double ssps_get_double       (STMT *stmt, ulong column_number, char *value,
+double ssps_get_double            (STMT *stmt, ulong column_number, char *value,
                                   ulong length);
 char *      ssps_get_string       (STMT *stmt, ulong column_number, char *value,
                                   ulong *length, char * buffer);
@@ -542,7 +513,7 @@ void free_connection_stmts(DBC *dbc);
 
 #define CHECK_DBC_OUTPUT(e, d) if(d == NULL) return set_env_error((ENV *)e, MYERR_S1009, NULL, 0)
 
-#define CHECK_STMT_OUTPUT(d, s) if(s == NULL) return set_conn_error((DBC *)d, MYERR_S1009, NULL, 0)
+#define CHECK_STMT_OUTPUT(d, s) if(s == NULL) return ((DBC *)d)->set_error(MYERR_S1009, NULL, 0)
 
 #define CHECK_DESC_OUTPUT(d, s) CHECK_STMT_OUTPUT(d, s)
 
