@@ -1,21 +1,21 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,14 +62,14 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
             break;
 
         case SQL_ATTR_CURSOR_TYPE:
-            if (((STMT *)Handle)->dbc->ds->force_use_of_forward_only_cursors)
+            if (((STMT *)Handle)->dbc->ds.opt_FORWARD_CURSOR)
             {
                 options->cursor_type= SQL_CURSOR_FORWARD_ONLY;
                 if (ValuePtr != (SQLPOINTER)SQL_CURSOR_FORWARD_ONLY)
                     return set_handle_error(HandleType,Handle,MYERR_01S02,
                                             "Forcing the use of forward-only cursor)",0);
             }
-            else if (((STMT *)Handle)->dbc->ds->dynamic_cursor)
+            else if (((STMT *)Handle)->dbc->ds.opt_DYNAMIC_CURSOR)
             {
                 if (ValuePtr != (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN)
                     options->cursor_type= (SQLUINTEGER)(SQLULEN)ValuePtr;
@@ -275,12 +275,12 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
           dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
           return SQL_SUCCESS;
         }
-        if (!(trans_supported(dbc)) || dbc->ds->disable_transactions)
-          return set_conn_error((DBC*)dbc,MYERR_S1C00,
-                                "Transactions are not enabled", 4000);
+        if (!(trans_supported(dbc)) || dbc->ds.opt_NO_TRANSACTIONS)
+          return ((DBC*)hdbc)->set_error(MYERR_S1C00,
+            "Transactions are not enabled", 4000);
 
         if (autocommit_on(dbc))
-          return odbc_stmt(dbc,"SET AUTOCOMMIT=0", SQL_NTS, TRUE);
+          return dbc->execute_query("SET AUTOCOMMIT=0", SQL_NTS, TRUE);
       }
       else if (!is_connected(dbc))
       {
@@ -288,7 +288,7 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
         return SQL_SUCCESS;
       }
       else if (trans_supported(dbc) && !(autocommit_on(dbc)))
-        return odbc_stmt(dbc,"SET AUTOCOMMIT=1", SQL_NTS, TRUE);
+        return dbc->execute_query("SET AUTOCOMMIT=1", SQL_NTS, TRUE);
       break;
 
     case SQL_ATTR_LOGIN_TIMEOUT:
@@ -296,7 +296,7 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
         /* we can't change timeout values in post connect state */
         if (is_connected(dbc))
         {
-          return set_conn_error(dbc, MYERR_S1011, NULL, 0);
+          return dbc->set_error(MYERR_S1011, NULL, 0);
         }
         else
         {
@@ -325,24 +325,25 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
     case SQL_ATTR_CURRENT_CATALOG:
       {
         char ldb[NAME_LEN+1], *db;
-        int cat_len= StringLengthPtr == SQL_NTS ?
+        size_t cat_len= StringLengthPtr == SQL_NTS ?
                      strlen((char *)ValuePtr) : StringLengthPtr;
 
         LOCK_DBC(dbc);
         if (cat_len > NAME_LEN)
         {
-          return set_conn_error(dbc, MYERR_01004,
-                                "Invalid string or buffer length", 0);
+          return dbc->set_error(MYERR_01004,
+            "Invalid string or buffer length", 0);
         }
 
         if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
-          return set_conn_error((DBC*)hdbc,MYERR_S1009,NULL, 0);
+          return dbc->set_error(MYERR_S1009,NULL, 0);
 
         if (is_connected(dbc))
         {
           if (mysql_select_db(dbc->mysql,(char*) db))
           {
-            set_conn_error(dbc,MYERR_S1000,mysql_error(dbc->mysql),mysql_errno(dbc->mysql));
+            dbc->set_error(MYERR_S1000, mysql_error(dbc->mysql),
+              mysql_errno(dbc->mysql));
             return SQL_ERROR;
           }
         }
@@ -352,10 +353,10 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
 
 
     case SQL_ATTR_ODBC_CURSORS:
-      if (dbc->ds->force_use_of_forward_only_cursors &&
+      if (dbc->ds.opt_FORWARD_CURSOR &&
         ValuePtr != (SQLPOINTER) SQL_CUR_USE_ODBC)
-        return set_conn_error((DBC*)hdbc,MYERR_01S02,
-                              "Forcing the Driver Manager to use ODBC cursor library",0);
+        return ((DBC*)hdbc)->set_error(MYERR_01S02,
+          "Forcing the Driver Manager to use ODBC cursor library",0);
       break;
 
     case SQL_OPT_TRACE:
@@ -365,8 +366,11 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
     case SQL_TRANSLATE_OPTION:
       {
         char buff[100];
-        sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",(int) Attribute);
-        return set_conn_error((DBC*)hdbc,MYERR_01S02,buff,0);
+        myodbc_snprintf(buff, sizeof(buff),
+                        "Suppose to set this attribute '%d' through driver "
+                        "manager, not by the driver",
+                        (int)Attribute);
+        return ((DBC*)hdbc)->set_error(MYERR_01S02, buff, 0);
       }
 
     case SQL_ATTR_PACKET_SIZE:
@@ -397,16 +401,16 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
           SQLRETURN rc;
           sprintf(buff,"SET SESSION TRANSACTION ISOLATION LEVEL %s",
                   level);
-          if (SQL_SUCCEEDED(rc = odbc_stmt(dbc, buff, SQL_NTS, TRUE)))
+          if (SQL_SUCCEEDED(rc = dbc->execute_query(buff, SQL_NTS, TRUE)))
           {
-            dbc->txn_isolation= (size_t)ValuePtr;
+            dbc->txn_isolation = (int)((size_t)ValuePtr);
           }
 
           return rc;
         }
         else
         {
-          return dbc->set_error( "HY024", "Invalid attribute value", 0);
+          return dbc->set_error("HY024", "Invalid attribute value", 0);
         }
       }
       break;
@@ -529,7 +533,7 @@ MySQLGetConnectAttr(SQLHDBC hdbc, SQLINTEGER attrib, SQLCHAR **char_attr,
     break;
 
   case SQL_ATTR_ODBC_CURSORS:
-    if (dbc->ds->force_use_of_forward_only_cursors)
+    if (dbc->ds.opt_FORWARD_CURSOR)
       *((SQLUINTEGER *)num_attr)= SQL_CUR_USE_ODBC;
     else
       *((SQLUINTEGER *)num_attr)= SQL_CUR_USE_IF_NEEDED;
@@ -557,9 +561,9 @@ MySQLGetConnectAttr(SQLHDBC hdbc, SQLINTEGER attrib, SQLCHAR **char_attr,
       }
 
       if (is_minimum_version(dbc->mysql->server_version, "8.0"))
-        result = odbc_stmt(dbc, "SELECT @@transaction_isolation", SQL_NTS, TRUE);
+        result = dbc->execute_query("SELECT @@transaction_isolation", SQL_NTS, true);
       else
-        result = odbc_stmt(dbc, "SELECT @@tx_isolation", SQL_NTS, TRUE);
+        result = dbc->execute_query("SELECT @@tx_isolation", SQL_NTS, true);
 
       if (result != SQL_SUCCESS)
       {
@@ -838,12 +842,12 @@ MySQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute, SQLPOINTER ValuePtr,
             break;
 
         case SQL_ATTR_PARAMSET_SIZE:
-            *(SQLUINTEGER *)ValuePtr= stmt->apd->array_size;
+            *(SQLUINTEGER *)ValuePtr = (SQLUINTEGER)stmt->apd->array_size;
             break;
 
         case SQL_ATTR_ROW_ARRAY_SIZE:
         case SQL_ROWSET_SIZE:
-            *(SQLUINTEGER *)ValuePtr= stmt->ard->array_size;
+            *(SQLUINTEGER *)ValuePtr = (SQLUINTEGER)stmt->ard->array_size;
             break;
 
         case SQL_ATTR_ROW_BIND_OFFSET_PTR:
