@@ -1,21 +1,21 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -492,7 +492,7 @@ SQLGetConnectAttrWImpl(SQLHDBC hdbc, SQLINTEGER attribute, SQLPOINTER value,
       see: "if (char_value)"
     */
     if (len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value_len)
       *value_len= len * sizeof(SQLWCHAR);
@@ -608,7 +608,7 @@ SQLGetDiagFieldW(SQLSMALLINT handle_type, SQLHANDLE handle,
 
     /* We set the error only when the result is intented to be returned */
     if (info && len > info_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc= dbc->set_error(MYERR_01004, NULL, 0);
 
     if (info_len)
       *info_len= (SQLSMALLINT)len * sizeof(SQLWCHAR);
@@ -696,7 +696,7 @@ SQLGetDiagRecWImpl(SQLSMALLINT handle_type, SQLHANDLE handle,
       and message_max is greaater than 0
     */
     if (message && message_max && len > message_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (message_len)
       *message_len= (SQLSMALLINT)len;
@@ -772,7 +772,7 @@ SQLGetInfoW(SQLHDBC hdbc, SQLUSMALLINT type, SQLPOINTER value,
       value is not NULL and value_max is 0
      */
     if (value && value_max && len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value_len)
       *value_len= (SQLSMALLINT)len * sizeof(SQLWCHAR);
@@ -827,7 +827,7 @@ SQLNativeSqlW(SQLHDBC hdbc, SQLWCHAR *in, SQLINTEGER in_len,
     *out_len= in_len;
 
   if (out && in_len >= out_max)
-    rc= set_conn_error((DBC *)hdbc, MYERR_01004, NULL, 0);
+    rc = ((DBC*)hdbc)->set_error(MYERR_01004, NULL, 0);
 
   if (out_max > 0)
   {
@@ -868,7 +868,9 @@ SQLPrepareWImpl(SQLHSTMT hstmt, SQLWCHAR *str, SQLINTEGER str_len,
     return stmt->set_error("22018", NULL, 0);
   }
 
-  return MySQLPrepare(hstmt, conv, str_len, true, false, force_prepare);
+  SQLRETURN rc = MySQLPrepare(hstmt, conv, str_len, false, force_prepare);
+  x_free(conv);
+  return rc;
 }
 
 
@@ -983,6 +985,8 @@ SQLProceduresW(SQLHSTMT hstmt,
 
   rc= MySQLProcedures(hstmt, catalog8, catalog_len, schema8, schema_len,
                       proc8, proc_len);
+  // Remove parameters
+  ((STMT *)hstmt)->free_reset_params();
 
   x_free(catalog8);
   x_free(schema8);
@@ -1339,8 +1343,8 @@ SQLBrowseConnectW(SQLHDBC hdbc, SQLWCHAR *in, SQLSMALLINT in_len,
 {
   CHECK_HANDLE(hdbc);
 
-  return set_conn_error((DBC*)hdbc,MYERR_S1000,
-                        "Driver does not support this API", 0);
+  return ((DBC*)hdbc)->set_error(MYERR_S1000,
+    "Driver does not support this API", 0);
 }
 
 
