@@ -1,21 +1,21 @@
-// Copyright (c) 2011, 2021, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +29,7 @@
 #include <thread>
 #include <vector>
 
-#include "odbctap.h"
+#include "odbc_util.h"
 #include "mysql_version.h"
 
 /*
@@ -886,7 +886,43 @@ DECLARE_TEST(t_fido_callback_test)
   return OK;
 }
 
+#ifndef WIN32
+/*
+  WL #15347 - Support MIT Kerberos on Win.
+  At this stage the kerberos testing requires complex
+  setup and manual testing. However, the failure testing
+  can be done in non-Windows platforms because they do not
+  support different kerberos authentication modes.
+*/
+DECLARE_TEST(t_kerberos_mode)
+{
+  try
+  {
+    // This should fail because only GSSAPI is supported in non-Windows platforms
+    odbc::connection conn(nullptr, nullptr, nullptr, nullptr,
+      "authentication-kerberos-mode=SSPI;");
+    return FAIL;
+  }
+  catch (odbc::Exception& ex)
+  {
+    is(ex.msg.find("Invalid value for authentication-kerberos-mode. "
+      "Only GSSAPI is supported.") !=
+      std::string::npos);
+  }
+
+  // This should succeed, but Kerberos is not enforced.
+  // Connector just checks the correctness of the option value.
+  odbc::connection conn(nullptr, nullptr, nullptr, nullptr,
+    "authentication-kerberos-mode=GSSAPI;");
+
+  return OK;
+}
+#endif
+
 BEGIN_TESTS
+#ifndef WIN32
+  ADD_TEST(t_kerberos_mode)
+#endif
   ADD_TEST(t_fido_callback_test)
   ADD_TEST(t_fido_test)
   // ADD_TEST(t_plugin_auth) TODO: Fix
