@@ -1,21 +1,21 @@
-// Copyright (c) 2018, 2014, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -196,6 +196,7 @@ DECLARE_TEST(t_bug70642)
     ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   }
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
   ok_sql(hstmt, "DROP PROCEDURE IF EXISTS t_bug70642");
 
   return OK;
@@ -246,7 +247,7 @@ DECLARE_TEST(t_bug17587913)
 {
   SQLHDBC hdbc1;
   SQLCHAR str[1024]={0};
-  SQLLEN len= 0;
+  SQLINTEGER len = 0;
   SQLCHAR *DatabaseName = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"\
                           "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"\
                           "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"\
@@ -267,7 +268,7 @@ DECLARE_TEST(t_bug17587913)
   get_connection(&hdbc1, NULL, NULL, NULL, DatabaseName, NULL);
 
   ok_con(hdbc1, SQLSetConnectAttr(hdbc1, SQL_ATTR_CURRENT_CATALOG,
-                                  DatabaseName, strlen(DatabaseName)));
+                                  DatabaseName, (SQLINTEGER)strlen(DatabaseName)));
 
   /* Expecting error here */
   SQLGetConnectAttr(hdbc1, SQL_ATTR_CURRENT_CATALOG, str, 100, &len);
@@ -331,6 +332,7 @@ DECLARE_TEST(t_bug17857204)
   }
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
 
   ok_sql(hstmt, "DROP TABLE bug17857204");
 
@@ -356,8 +358,6 @@ DECLARE_TEST(t_bug17854697)
                       "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
   SQLCHAR buf[1024]= {0};
 
-  int len= strlen(any_name);
-
   /* lets check all catalog functions */
   expect_stmt(hstmt, SQLColumnPrivileges(hstmt, any_name, SQL_NTS, NULL, 0,
                                          any_name, SQL_NTS, any_name,
@@ -624,6 +624,7 @@ DECLARE_TEST(t_bug18325878)
   }
 
   ok_stmt(hstmt, SQLExecute(hstmt));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
   ok_sql(hstmt, "SELECT * FROM t_bug18325878");
 
   while (SQLFetch(hstmt) == SQL_SUCCESS)
@@ -954,6 +955,7 @@ DECLARE_TEST(t_setpos_update_no_ssps)
   ok_stmt(hstmt1, SQLSetPos(hstmt1, 1, SQL_UPDATE, SQL_LOCK_NO_CHANGE));
 
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_RESET_PARAMS));
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt, "drop table if exists t_setpos_update_no_ssps");
@@ -1034,7 +1036,6 @@ DECLARE_TEST(t_bug32813838)
     SQLCHAR name[16];
   } rows[25];
   size_t row_size= (sizeof(rows) / 25);
-  SQLINTEGER out_id, out_x;
   SQLULEN bind_offset= 20 * row_size;
   SQLHANDLE ipd = NULL;
   SQLHANDLE apd = NULL;
