@@ -1,27 +1,33 @@
-/*
-  Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// https://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
 
+#include <time.h>
 #include "odbctap.h"
 
 
@@ -30,9 +36,18 @@ DECLARE_TEST(my_ts)
   SQLCHAR          szTs[50];
   TIMESTAMP_STRUCT ts;
   SQLLEN           len;
+  int is_fraction_capable = mysql_min_version(hdbc, "5.6.", 4);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS my_ts");
-  ok_sql(hstmt, "CREATE TABLE my_ts (ts TIMESTAMP)");
+
+  if(is_fraction_capable)
+  {
+    ok_sql(hstmt, "CREATE TABLE my_ts (ts TIMESTAMP(6))");
+  }
+  else
+  {
+    ok_sql(hstmt, "CREATE TABLE my_ts (ts TIMESTAMP)");
+  }
 
   /* insert using SQL_C_CHAR to SQL_TIMESTAMP */
   strcpy((char *)szTs, "2002-01-07 10:20:49.06");
@@ -54,7 +69,8 @@ DECLARE_TEST(my_ts)
   ts.hour= 19;
   ts.minute= 47;
   ts.second= 59;
-  ts.fraction= 4;
+  ts.fraction= is_fraction_capable ? 123456000 : 0;
+
   ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
                                   SQL_C_TIMESTAMP, SQL_TIMESTAMP,
                                   0, 0, &ts, sizeof(ts), NULL));
@@ -71,8 +87,11 @@ DECLARE_TEST(my_ts)
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 1));
 
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szTs, sizeof(szTs), &len));
-  is_str(szTs, "2002-01-07 10:20:49", len);
-  printf("# row1 using SQL_C_CHAR: %s (%ld)\n", szTs, len);
+
+  is_str(szTs, is_fraction_capable ? "2002-01-07 10:20:49.060000" :
+                                     "2002-01-07 10:20:49", len);
+
+  printf("# row1 using SQL_C_CHAR: %s (%ld)\n", szTs, (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 1));
 
@@ -83,15 +102,18 @@ DECLARE_TEST(my_ts)
   is_num(ts.hour,  10);
   is_num(ts.minute,20);
   is_num(ts.second,49);
+  is_num(ts.fraction, is_fraction_capable ? 60000000 : 0);
+
   printf("# row1 using SQL_C_TIMESTAMP: %d-%d-%d %d:%d:%d.%d (%ld)\n",
          ts.year, ts.month,ts.day, ts.hour, ts.minute, ts.second, ts.fraction,
-         len);
+         (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 2));
 
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szTs, sizeof(szTs), &len));
-  is_str(szTs, "2002-01-07 19:47:59", len);
-  printf("# row2 using SQL_C_CHAR: %s(%ld)\n", szTs, len);
+  is_str(szTs, is_fraction_capable ? "2002-01-07 19:47:59.123456" :
+                                      "2002-01-07 19:47:59", len);
+  printf("# row2 using SQL_C_CHAR: %s(%ld)\n", szTs, (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 2));
 
@@ -102,9 +124,11 @@ DECLARE_TEST(my_ts)
   is_num(ts.hour,  19);
   is_num(ts.minute,47);
   is_num(ts.second,59);
+  is_num(ts.fraction, is_fraction_capable ? 123456000 : 0);
+
   printf("# row2 using SQL_C_TIMESTAMP: %d-%d-%d %d:%d:%d.%d (%ld)\n",
          ts.year, ts.month,ts.day, ts.hour, ts.minute, ts.second, ts.fraction,
-         len);
+         (long)len);
 
 
   expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 3),
@@ -115,75 +139,90 @@ DECLARE_TEST(my_ts)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS my_ts");
 
+  /* Test of 2-digits year(YYMMDD) format */
+  ok_sql(hstmt, "select \"910825\"");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TIMESTAMP, &ts, sizeof(ts), &len));
+
+  is_num(1991, ts.year);
+  is_num(8, ts.month);
+  is_num(25, ts.day);
+
   return OK;
 }
 
 
 DECLARE_TEST(t_tstotime)
 {
-    SQLRETURN rc;
-    SQL_TIMESTAMP_STRUCT ts;
+  SQL_TIMESTAMP_STRUCT ts, ts1, ts2;
+  int is_fraction_capable = mysql_min_version(hdbc, "5.6.", 4);
 
-    ts.day    = 02;
-    ts.month  = 8;
-    ts.year   = 2001;
-    ts.hour   = 18;
-    ts.minute = 20;
-    ts.second = 45;
-    ts.fraction = 05;   
+  ts.day    = 02;
+  ts.month  = 8;
+  ts.year   = 2001;
+  ts.hour   = 18;
+  ts.minute = 20;
+  ts.second = 45;
+  ts.fraction = is_fraction_capable ? 555000 : 0;
+
+  memcpy(&ts1, (void*) &ts, sizeof(SQL_TIMESTAMP_STRUCT));
+
+  memcpy(&ts2, (void*) &ts1, sizeof(SQL_TIMESTAMP_STRUCT));
+
+  /*
+    SQL_TIME cannot have the fractional part
+    http://msdn.microsoft.com/en-us/library/ms709385%28v=vs.85%29.aspx
+  */
+  ts1.fraction= 0;
+
+  /* Same for SQL_DATE - time is truncated -> error */
+  ts2.fraction= ts2.hour= ts2.minute= ts2.second= 0;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_tstotime");
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_con(hdbc, SQLTransact(NULL,hdbc,SQL_COMMIT));
 
-    rc = tmysql_exec(hstmt,"create table t_tstotime(col1 date ,col2 time, col3 timestamp(14))");
-    mystmt(hstmt,rc);
+  if(is_fraction_capable)
+  {
+    ok_sql(hstmt,"create table t_tstotime(col1 date,col2 time, col3 timestamp(6))");
+  }
+  else
+  {
+    ok_sql(hstmt,"create table t_tstotime(col1 date ,col2 time, col3 timestamp)");
+  }
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_con(hdbc, SQLTransact(NULL,hdbc,SQL_COMMIT));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
-    /* TIMESTAMP TO DATE, TIME and TS CONVERSION */
-    rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_tstotime(col1,col2,col3) values(?,?,?)",SQL_NTS);
-    mystmt(hstmt,rc);   
+  /* TIMESTAMP TO DATE, TIME and TS CONVERSION */
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"insert into t_tstotime(col1,col2,col3) values(?,?,?)",SQL_NTS));
 
-    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_DATE,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                                  SQL_DATE,0,0,&ts2,sizeof(ts2),NULL));
 
-    rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_TIME,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  //ts1.fraction= 0;
+  ok_stmt(hstmt, SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                                  SQL_TIME,0,0,&ts1,sizeof(ts1),NULL));
 
-    rc = SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                                  SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL));
 
-    rc = SQLExecute(hstmt);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLExecute(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    mystmt(hstmt,rc);  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_RESET_PARAMS));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_con(hdbc, SQLTransact(NULL,hdbc,SQL_COMMIT));
 
-    rc = tmysql_exec(hstmt,"select * from t_tstotime");
-    mystmt(hstmt,rc);  
+  ok_sql(hstmt,"select * from t_tstotime");
 
-    my_assert( 1 == myresult(hstmt));
+  my_assert( 1 == myresult(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_UNBIND));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_tstotime");
 
@@ -207,12 +246,30 @@ DECLARE_TEST(t_tstotime1)
                             (SQLCHAR *)"INSERT INTO t_tstotime1 VALUES (?,?,?)",
                             SQL_NTS));
 
-  ok_stmt(hstmt, SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_DATE,0,0,&ts,sizeof(ts),NULL));
-  ok_stmt(hstmt, SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_TIME,0,0,&ts,sizeof(ts),NULL));
-  ok_stmt(hstmt, SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIME, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIMESTAMP, 0, 0, &ts, sizeof(ts), NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* Taking only date part */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_DATE, 0, 0, &ts, 10, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* are not taking fractional part */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIME, 0, 0, &ts, 19, NULL));
 
   ok_stmt(hstmt, SQLExecute(hstmt));
 
@@ -286,57 +343,48 @@ DECLARE_TEST(t_time)
   SQLCHAR         str[20];
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_time");
-    rc = tmysql_exec(hstmt,"create table t_time(tm time, ts timestamp(14))");
-    mystmt(hstmt,rc);
+  rc = tmysql_exec(hstmt,"create table t_time(tm time, ts timestamp)");
+  mystmt(hstmt,rc);
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  // Disable strict mode and no zero dates
+  ok_sql(hstmt, "set session sql_mode=''");
 
-    ok_stmt(hstmt, SQLPrepare(hstmt,
-                              (SQLCHAR *)"insert into t_time values (?,?)",
-                              SQL_NTS));
+  ok_con(hdbc, SQLTransact(NULL,hdbc,SQL_COMMIT));
 
-    rc = SQLBindParameter( hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
-                           SQL_TIME, 0, 0, &tm, 0, NULL );
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"insert into t_time values (?,?)",
+                            SQL_NTS));
 
-    rc = SQLBindParameter( hstmt, 2, SQL_PARAM_INPUT, SQL_C_TIME,
-                           SQL_TIMESTAMP, 0, 0, &tm, 15, NULL );
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLBindParameter( hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                         SQL_TIME, 0, 0, &tm, 0, NULL ));
 
-    tm.hour = 20;
-    tm.minute = 59;
-    tm.second = 45;
+  ok_stmt(hstmt, SQLBindParameter( hstmt, 2, SQL_PARAM_INPUT, SQL_C_TIME,
+                         SQL_TIMESTAMP, 0, 0, &tm, 15, NULL ));
 
-    rc = SQLExecute(hstmt);
-    mystmt(hstmt,rc);
+  tm.hour = 20;
+  tm.minute = 59;
+  tm.second = 45;
 
-    rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLExecute(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_con(hdbc, SQLTransact(NULL,hdbc,SQL_COMMIT));
 
-    ok_sql(hstmt, "select tm from t_time");
+  ok_sql(hstmt, "select tm from t_time");
 
-    rc = SQLFetch(hstmt);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFetch(hstmt));
 
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,&str,100,NULL);
-    mystmt(hstmt,rc);
-    is_str(str, "20:59:45", 9);
+  ok_stmt(hstmt, SQLGetData(hstmt,1,SQL_C_CHAR,&str,100,NULL));
+  is_str(str, "20:59:45", 9);
 
-    rc = SQLFetch(hstmt);
-    my_assert(rc == SQL_NO_DATA_FOUND);
+  rc = SQLFetch(hstmt);
+  my_assert(rc == SQL_NO_DATA_FOUND);
 
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_UNBIND));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_time");
 
@@ -602,6 +650,9 @@ DECLARE_TEST(t_bug12520)
   SQLLEN len, my_time_cb;
   SQLCHAR datetime[50];
 
+  // Disable the strict SQL mode and zero date values
+  ok_sql(hstmt, "set session sql_mode='' ");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug12520");
   ok_sql(hstmt,
          "CREATE TABLE t_bug12520 (a DATETIME DEFAULT '0000-00-00 00:00',"
@@ -637,6 +688,9 @@ DECLARE_TEST(t_bug15773)
   SQL_DATE_STRUCT a,b,c,d;
   SQLLEN len1;
 
+  // Disable the strict SQL mode and zero date values
+  ok_sql(hstmt, "set session sql_mode='' ");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15773");
   ok_sql(hstmt, "CREATE TABLE t_bug15773("
          "`a` varchar(255) NOT NULL default '',"
@@ -666,6 +720,7 @@ DECLARE_TEST(t_bug15773)
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15773");
   return OK;
@@ -679,6 +734,9 @@ DECLARE_TEST(t_bug9927)
 {
   SQLCHAR col[10];
 
+  // Disable the strict SQL mode and zero date values
+  ok_sql(hstmt, "set session sql_mode='' ");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug9927");
   ok_sql(hstmt,
          "CREATE TABLE t_bug9927 (a TIMESTAMP DEFAULT 0,"
@@ -711,6 +769,9 @@ DECLARE_TEST(t_bug30081)
   if (!mysql_min_version(hdbc, "5.1.23", 6))
     skip("necessary feature added in MySQL 5.1.23");
 
+  // Disable the strict SQL mode and zero date values
+  ok_sql(hstmt, "set session sql_mode='' ");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug30081");
   ok_sql(hstmt,
          "CREATE TABLE t_bug30081 (a TIMESTAMP DEFAULT 0,"
@@ -749,24 +810,28 @@ DECLARE_TEST(t_datecolumns)
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   is_str(my_fetch_str(hstmt, col, 4), "a", 1);
+  is_num(my_fetch_int(hstmt, 5), SQL_TYPE_TIMESTAMP);
   is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
   is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIMESTAMP);
 
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   is_str(my_fetch_str(hstmt, col, 4), "b", 1);
+  is_num(my_fetch_int(hstmt, 5), SQL_TYPE_TIMESTAMP);
   is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
   is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIMESTAMP);
 
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   is_str(my_fetch_str(hstmt, col, 4), "c", 1);
+  is_num(my_fetch_int(hstmt, 5), SQL_TYPE_DATE);
   is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
   is_num(my_fetch_int(hstmt, 15), SQL_TYPE_DATE);
 
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   is_str(my_fetch_str(hstmt, col, 4), "d", 1);
+  is_num(my_fetch_int(hstmt, 5), SQL_TYPE_TIME);
   is_num(my_fetch_int(hstmt, 14), SQL_DATETIME);
   is_num(my_fetch_int(hstmt, 15), SQL_TYPE_TIME);
 
@@ -804,6 +869,9 @@ DECLARE_TEST(t_bug14414)
   SQLCHAR col[10];
   SQLSMALLINT nullable;
 
+  if (!mysql_min_version(hdbc, "8.0.0", 5))
+    skip("MySQL 8.0 or newer required");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug14414");
   ok_sql(hstmt, "CREATE TABLE t_bug14414(a TIMESTAMP, b TIMESTAMP NOT NULL,"
         "c TIMESTAMP NULL)");
@@ -818,8 +886,8 @@ DECLARE_TEST(t_bug14414)
 
   ok_stmt(hstmt, SQLFetch(hstmt));
   is_str(my_fetch_str(hstmt, col, 4), "b", 1);
-  is_num(my_fetch_int(hstmt, 11), SQL_NO_NULLS);
-  is_str(my_fetch_str(hstmt, col, 18), "NO", 3);
+  is_num(my_fetch_int(hstmt, 11), SQL_NULLABLE);
+  is_str(my_fetch_str(hstmt, col, 18), "YES", 3);
 
   ok_stmt(hstmt, SQLFetch(hstmt));
   is_str(my_fetch_str(hstmt, col, 4), "c", 1);
@@ -842,7 +910,7 @@ DECLARE_TEST(t_bug14414)
 
   ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, col, sizeof(col), NULL, NULL, NULL,
                                 NULL, &nullable));
-  is_num(nullable, SQL_NO_NULLS);
+  is_num(nullable, SQL_NULLABLE);
 
   ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, col, sizeof(col), NULL, NULL, NULL,
                                 NULL, &nullable));
@@ -949,7 +1017,11 @@ DECLARE_TEST(t_bug37342)
   ts.hour= 19;
   ts.minute= 47;
   ts.second= 59;
-  ts.fraction= 4;
+  ts.fraction= 4000;
+
+  /* Fractional truncation */
+  expect_sql(hstmt, "SELECT ? AS foo", SQL_ERROR);
+  ts.fraction= 0;
 
   ok_sql(hstmt, "SELECT ? AS foo");
 
@@ -965,22 +1037,424 @@ DECLARE_TEST(t_bug37342)
 }
 
 
+/**
+  Bug#60646 - Fractions of seconds ignored in results
+*/
+DECLARE_TEST(t_bug60646)
+{
+  SQLCHAR buff[128];
+  TIMESTAMP_STRUCT ts;
+  SQLLEN len;
+  const char *expected= "2012-01-01 01:01:01.000001";
+
+  ok_sql(hstmt,
+        "SELECT timestamp('2012-01-01 01:01:01.000001')"            /*1*/
+        " ,timestamp('2012-01-01 01:01:01.100002')"                 /*2*/
+        " ,'2011-07-29 17:52:15.0000000009'"                        /*3*/
+        " ,'1000-01-01 12:00:00.000000001'"                         /*4*/
+        " ,time('2011-12-31 23:59:59.999999')"                      /*5*/
+        " ,ADDTIME('9999-12-31 23:59:59.999999', '1 1:1:1.000002')" /*6*/
+        );
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* Fields 1-4 checking conversions from date as a string
+  /* 1) just to be sure that everything is fine with string */
+  is_str(my_fetch_str(hstmt, buff, 1), expected, sizeof(expected));
+
+  /* 2) testing if fractional part is converted to nanoseconds correctly */
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+
+  is_num(ts.fraction, 100002000);
+
+  /* 3) fractional part is less than we care (less than nanosecond).
+        Test using string as MySQL does not support units less than a microsecond */
+  ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+  is_num(ts.fraction, 0);
+
+  /* 4) testing if min fraction detected
+        Again - mysql supports microseconds only. thus using string
+   */
+  ok_stmt(hstmt, SQLGetData(hstmt, 4, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+  is_num(ts.fraction, 1);
+
+  /* 5) if time is converted to timestamp - checking if current date is set
+        and if fractional part in place. former can actually fail if day is
+        being changed */
+
+  {
+    time_t sec_time= time(NULL);
+    struct tm * cur_tm;
+
+    ok_stmt(hstmt, SQLGetData(hstmt, 5, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                              &len));
+    cur_tm= localtime(&sec_time);
+
+    is_num(ts.year, 1900 + cur_tm->tm_year);
+    is_num(ts.month, 1 + cur_tm->tm_mon);
+    is_num(ts.day, cur_tm->tm_mday);
+  }
+
+  is_num(ts.fraction, 999999000);
+
+  /* 6) Expecting an error because of longer date
+        At the moment ADDTIME('9999-12-31 23:59:59.999999', '1 1:1:1.000002')
+        will give you 10000-01-02 01:01:01.000001
+        Fixed in 5.6
+   */
+
+  if (!mysql_min_version(hdbc, "5.6.", 4))
+  {
+    expect_stmt(hstmt, SQLGetData(hstmt, 6, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                              &len), SQL_ERROR);
+
+    if (check_sqlstate(hstmt, "22018") != OK)
+    {
+      return FAIL;
+    }
+  }
+
+  /* 5th col once again This time we get it in time struct. Thus we are
+     loosing fractioanl part. Thus the state has to be 01S07 and
+     SQL_SUCCESS_WITH_INFO returned */
+  {
+    SQL_TIME_STRUCT timestruct;
+
+    expect_stmt(hstmt, SQLGetData(hstmt, 5, SQL_C_TYPE_TIME, &timestruct,
+                            sizeof(timestruct), &len), SQL_SUCCESS_WITH_INFO);
+
+    if (check_sqlstate(hstmt, "01S07") != OK)
+    {
+      return FAIL;
+    }
+  }
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  return OK;
+}
+
+
+/* Bug#60648 ODBC prepared statements ignore fractional part of temporal data
+   types */
+DECLARE_TEST(t_bug60648)
+{
+  SQL_TIMESTAMP_STRUCT param, result;
+
+  param.year=     2011;
+  param.month=    8;
+  param.day=      6;
+  param.hour=     1;
+  param.minute=   2;
+  param.second=   3;
+  param.fraction= 1000;
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"select ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_DATE, 0, 0, &param, 0, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_TIME, 0, 0, &param, 0, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_TIMESTAMP, 0, 0, &param, 0, NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TYPE_TIMESTAMP, &result, 0,
+                            NULL));
+
+  is_num(1000, result.fraction);
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_b13975271)
+{
+  if (!mysql_min_version(hdbc, "5.6.", 4))
+  {
+    skip("Necessary feature added in MySQL 5.6.*");
+  }
+  else
+  {
+    SQLCHAR ts[27];
+    SQLLEN len;
+
+    ok_sql(hstmt, "DROP TABLE IF EXISTS t_b13975271");
+    ok_sql(hstmt, "CREATE TABLE t_b13975271 (ts TIMESTAMP(6), dt DATETIME(6),\
+                    t TIME(6))");
+
+    strcpy((char *)ts, "2012-04-25 10:20:49.0194");
+
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_TIMESTAMP,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_CHAR,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_CHAR,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLPrepare(hstmt, "INSERT INTO t_b13975271(ts,dt,t) \
+                                      VALUES (?,?,?)",
+                              SQL_NTS));
+    ok_stmt(hstmt, SQLExecute(hstmt));
+
+    ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+    ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+
+    /* now fetch and verify the results .. */
+    ok_sql(hstmt, "SELECT * FROM t_b13975271");
+
+    ok_stmt(hstmt, SQLFetch(hstmt));
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "2012-04-25 10:20:49.019400", 26);
+
+    /*To make sure that for next test we compare not the data from prev call */
+    ts[0]='\0';
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "2012-04-25 10:20:49.0194", 24);
+    ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "10:20:49.0194", 13);
+
+    ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+    ok_sql(hstmt, "DROP TABLE IF EXISTS t_b13975271");
+  }
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_17613161)
+{
+  SQL_TIME_STRUCT ts, result;
+  SQL_INTERVAL_STRUCT h2s, interval;
+
+  h2s.intval.day_second.hour=   ts.hour  = 100;
+  h2s.intval.day_second.minute= ts.minute= 20;
+  h2s.intval.day_second.second= ts.second= 45;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161");
+
+  ok_sql(hstmt, "CREATE TABLE t_17613161(col1 time)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"INSERT INTO t_17613161 "
+                            "(col1) VALUES (?)",SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                                  SQL_TIME, 0, 0, &ts, sizeof(ts), NULL));
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* Such conversion is not supported */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                                  SQL_INTERVAL_HOUR_TO_SECOND, 0, 0, &ts, sizeof(ts), NULL));
+ expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+ is_num(check_sqlstate(hstmt, "07006"), OK);
+
+  /* For interval types big hours should work fine */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_INTERVAL_HOUR_TO_SECOND,
+                                  SQL_INTERVAL_HOUR_TO_SECOND, 0, 0, &h2s, sizeof(h2s), NULL));
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+
+  ok_sql(hstmt,"SELECT * FROM t_17613161");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TYPE_TIME, &result, 0,
+                            NULL), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22007"), OK);
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_MINUTE, &interval, 0,
+                        NULL), SQL_SUCCESS_WITH_INFO);
+  is_num(check_sqlstate(hstmt, "01S07"), OK);
+
+  is_num(interval.intval.day_second.second, 0);
+  is_num(interval.intval.day_second.minute, 20);
+  is_num(interval.intval.day_second.hour, 100);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_SECOND, &interval, 0,
+                        NULL));
+
+  is_num(interval.intval.day_second.second, 45);
+  is_num(interval.intval.day_second.minute, 20);
+  is_num(interval.intval.day_second.hour, 100);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_17613161_bookmark)
+{
+  SQLCHAR bData[2][10];
+  SQL_TIME_STRUCT tm[2]= {0};
+  SQL_INTERVAL_STRUCT h2s[2];
+
+  h2s[0].intval.day_second.hour=   tm[0].hour   = 11;
+  h2s[0].intval.day_second.minute= tm[0].minute = 02;
+  h2s[0].intval.day_second.second= tm[0].second = 19;
+
+  h2s[1].intval.day_second.hour=   tm[1].hour  = 100;
+  h2s[1].intval.day_second.minute= tm[1].minute= 20;
+  h2s[1].intval.day_second.second= tm[1].second= 45;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161_bookmark");
+
+  ok_sql(hstmt,"CREATE TABLE t_17613161_bookmark(col1 time)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)2, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CONCURRENCY,
+                                (SQLPOINTER)SQL_CONCUR_ROWVER, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS,
+                                (SQLPOINTER) SQL_UB_VARIABLE, 0));
+
+  ok_sql(hstmt, "SELECT * FROM t_17613161_bookmark");
+
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 0),
+              SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 0, SQL_C_VARBOOKMARK, bData,
+                            sizeof(bData[0]), NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_TYPE_TIME, tm,
+                            sizeof(tm[0]), NULL));
+
+  expect_stmt(hstmt, SQLBulkOperations(hstmt, SQL_ADD), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_SECOND, h2s,
+                            sizeof(h2s[0]), NULL));
+  ok_stmt(hstmt, SQLBulkOperations(hstmt, SQL_ADD));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)1, 0));
+
+  ok_sql(hstmt, "SELECT * FROM t_17613161_bookmark");
+  is_num(myrowcount(hstmt), 2);
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(atol(bData[0]), 1);
+  is_num(tm[0].hour, 11);
+  is_num(tm[0].minute, 02);
+  is_num(tm[0].second, 19);
+
+  is_num(atol(bData[1]), 2);
+  is_num(tm[1].hour, 100);
+  is_num(tm[1].minute, 20);
+  is_num(tm[1].second, 45);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161_bookmark");
+
+  return OK;
+}
+
+
+/**
+ Bug #25386024: Error "Date Overflow"
+*/
+DECLARE_TEST(t_date_overflow)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+           NULL, NULL, NULL, "NO_DATE_OVERFLOW=1"));
+
+  char *query_prep = "INSERT INTO t_date_overflow (a) VALUES (?)";
+  SQL_TIMESTAMP_STRUCT ts;
+
+  ts.day = 14;
+  ts.month = 2;
+  ts.year = 2017;
+  ts.hour = 16;
+  ts.minute = 11;
+  ts.second = 45;
+  ts.fraction = 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_date_overflow");
+  ok_sql(hstmt, "CREATE TABLE t_date_overflow (a DATE)");
+
+  /* By default the INSERT should end with the error */
+  ok_stmt(hstmt, SQLPrepare(hstmt, query_prep, SQL_NTS));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIMESTAMP,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+
+  /* Empty result is expected */
+  ok_sql(hstmt, "SELECT * FROM t_date_overflow");
+  is_num(my_print_non_format_result(hstmt), 0);
+
+  /* The INSERT with NO_DATE_OVERFLOW=1 should be successful */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, query_prep, SQL_NTS));
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_TIMESTAMP,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  /* One row is expected */
+  ok_sql(hstmt, "SELECT * FROM t_date_overflow");
+  is_num(my_print_non_format_result(hstmt), 1);
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_date_overflow");
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
+
+
 BEGIN_TESTS
+  // ADD_TEST(t_bug60646)
+  ADD_TEST(t_bug37342)
+  ADD_TEST(t_bug14414)
+  ADD_TEST(t_bug30081)
+  ADD_TEST(t_bug9927)
+  ADD_TEST(t_bug15773)
+  ADD_TEST(t_bug12520)
+  ADD_TEST(t_time)
+  ADD_TEST(t_date_overflow)
   ADD_TEST(my_ts)
   ADD_TEST(t_tstotime)
   ADD_TEST(t_tstotime1)
-  ADD_TEST(t_bug25846)
-  ADD_TEST(t_time)
-  ADD_TEST(t_time1)
-  ADD_TEST(t_bug12520)
-  ADD_TEST(t_bug15773)
-  ADD_TEST(t_bug9927)
-  ADD_TEST(t_bug30081)
   ADD_TEST(t_datecolumns)
-  ADD_TEST(t_bug14414)
-  ADD_TEST(t_bug30939)
   ADD_TEST(t_bug31009)
-  ADD_TEST(t_bug37342)
+  ADD_TEST(t_17613161)
+#ifndef USE_IODBC
+  ADD_TEST(t_17613161_bookmark)
+#endif
+  ADD_TEST(t_bug25846)
+  ADD_TEST(t_time1)
+  ADD_TEST(t_bug30939)
+  ADD_TEST(t_bug60648)
+  ADD_TEST(t_b13975271)
 END_TESTS
 
 
