@@ -1,26 +1,31 @@
-/*
-  Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// https://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
 /*
  * Tests for descriptors.
  */
@@ -66,7 +71,7 @@ DECLARE_TEST(t_desc_paramset)
 
   /* set the fields */
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_PARAMSET_SIZE,
-                                (SQLPOINTER) parsetsize, 0));
+                                (SQLPOINTER)((size_t)parsetsize), 0));
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_PARAM_STATUS_PTR,
                                 parstatus, 0));
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_PARAM_OPERATION_PTR,
@@ -322,43 +327,54 @@ DECLARE_TEST(t_explicit_error)
                                 &desc1, 0, NULL));
   ok_stmt(hstmt2, SQLGetStmtAttr(hstmt2, SQL_ATTR_APP_ROW_DESC,
                                  &desc2, 0, NULL));
+  printf("LINE:%d\n",__LINE__);
+
+#ifndef SQL_ATTR_UNIXODBC_VERSION
+  /* This call is checked by the driver manager and sometimes UnixODBC
+     crashes in there.
+  */
 
   /* can't set implicit ard from a different statement */
   expect_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_APP_ROW_DESC,
                                     desc2, 0), SQL_ERROR);
   is(check_sqlstate(hstmt, "HY017") == OK);
-
+#endif
   /* can set it to the same statement */
+  printf("LINE:%d\n",__LINE__);
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_APP_ROW_DESC, desc1, 0));
+  printf("LINE:%d\n",__LINE__);
 
   /* can't set implementation descriptors */
   expect_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_IMP_ROW_DESC,
                                     desc1, 0), SQL_ERROR);
+  printf("LINE:%d\n",__LINE__);
   is(check_sqlstate(hstmt, "HY024") == OK ||
      check_sqlstate(hstmt, "HY017") == OK);
+  printf("LINE:%d\n",__LINE__);
 
   /*
     can't free implicit descriptors
     This crashes unixODBC 2.2.11, as it improperly frees the descriptor,
     and again tries to when freeing the statement handle.
   */
-  if (using_unixodbc_version(henv, "2.2.11"))
-  {
+#ifdef SQL_ATTR_UNIXODBC_VERSION
     printMessage("free implicit descriptor test skipped under unixODBC 2.2.11");
-  }
-  else
-  {
+#else
     expect_desc(desc1, SQLFreeHandle(SQL_HANDLE_DESC, desc1), SQL_ERROR);
     is(check_sqlstate_ex(desc1, SQL_HANDLE_DESC, "HY017") == OK);
-  }
+#endif
 
   /* can't set apd as ard (and vice-versa) */
+  printf("LINE:%d\n",__LINE__);
   ok_con(hdbc, SQLAllocHandle(SQL_HANDLE_DESC, hdbc, &expapd));
+  printf("LINE:%d\n",__LINE__);
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_APP_PARAM_DESC,
                                 expapd, 0)); /* this makes expapd an apd */
 
+  printf("LINE:%d\n",__LINE__);
   expect_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_APP_ROW_DESC,
                                     expapd, 0), SQL_ERROR);
+  printf("LINE:%d\n",__LINE__);
   is(check_sqlstate(hstmt, "HY024") == OK);
 
   /*
@@ -366,9 +382,12 @@ DECLARE_TEST(t_explicit_error)
     Even though the above call failed, unixODBC saved this value internally
     and returns it. desc1 should *not* be the same as the explicit apd
   */
+  printf("LINE:%d\n",__LINE__);
   ok_stmt(hstmt, SQLGetStmtAttr(hstmt, SQL_ATTR_APP_ROW_DESC,
                                 &desc1, 0, NULL));
+  printf("LINE:%d\n",__LINE__);
   printMessage("explicit apd: %x, stmt's ard: %x", expapd, desc1);
+  printf("LINE:%d\n",__LINE__);
 
   return OK;
 }
@@ -381,7 +400,10 @@ DECLARE_TEST(t_explicit_error)
 DECLARE_TEST(t_mult_stmt_free)
 {
 #define mult_count 3
-  SQLHANDLE expard, expapd,desc;
+  SQLHANDLE expard, expapd;
+#ifndef _WIN32
+  SQLHANDLE desc;
+#endif
   SQLHANDLE stmt[mult_count];
   SQLINTEGER i;
   SQLINTEGER imp_params[mult_count];
@@ -586,18 +608,86 @@ DECLARE_TEST(t_bug44576)
 }
 
 
+/* If no default database is selected for the connection, call of SQLColumns
+   causes error "Unknown database 'null'" */
+DECLARE_TEST(t_desc_curcatalog)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLCHAR conn_in[512];
+  SQLHANDLE ird;
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, USE_DRIVER,
+                                        NULL, NULL, "", NULL));
+
+  ok_sql(hstmt1, "select 10 AS no_catalog_column");
+
+  ok_stmt(hstmt1, SQLGetStmtAttr(hstmt1, SQL_ATTR_IMP_ROW_DESC, &ird, 0, NULL));
+  ok_desc(ird, SQLGetDescField(ird, 1, SQL_DESC_CATALOG_NAME, conn_in,
+                               sizeof(conn_in), NULL));
+
+
+  is(conn_in==NULL);
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
+
+/*
+  Bug #18636600 - SQLGetData with invalid column number crashes
+*/
+DECLARE_TEST(t_bug18636600)
+{
+  char buf[255];
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS, (SQLPOINTER)SQL_UB_VARIABLE, 0));
+  ok_sql(hstmt, "select 1");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  expect_stmt(hstmt, SQLGetData(hstmt, -1, SQL_C_VARBOOKMARK, buf, sizeof(buf), NULL),
+              SQL_ERROR);
+  return OK;
+}
+
+
+/*
+Bug #18641633 - Assert failure in SQLSetDescField() API
+*/
+DECLARE_TEST(t_bug18641633)
+{
+  char buf[255];
+  SQLHANDLE ard = NULL;
+  ok_stmt(hstmt, SQLGetStmtAttr(hstmt, SQL_ATTR_APP_ROW_DESC,
+    &ard, SQL_IS_POINTER, NULL));
+  ok_sql(hstmt, "select 1");
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, buf, sizeof(buf), NULL));
+  /* Result does not matter, it should not crash */
+  SQLSetDescField(ard, 1, SQL_DESC_COUNT, (SQLPOINTER)0, SQL_IS_SMALLINT);
+  return OK;
+}
+
+
+DECLARE_TEST(dummy_test)
+{
+  return OK;
+}
+
 BEGIN_TESTS
   ADD_TODO(t_desc_paramset)
   ADD_TEST(t_desc_set_error)
   ADD_TEST(t_sqlbindcol_count_reset)
   ADD_TEST(t_desc_default_type)
   ADD_TEST(t_basic_explicit)
+#ifndef USE_IODBC
   ADD_TEST(t_explicit_error)
+#endif
   ADD_TEST(t_mult_stmt_free)
   ADD_TEST(t_set_null_use_implicit)
   ADD_TEST(t_free_stmt_with_exp_desc)
   ADD_TEST(t_bug41081)
   ADD_TEST(t_bug44576)
+  ADD_TEST(t_bug18641633)
+  ADD_TEST(t_bug18636600)
+  // ADD_TODO(t_desc_curcatalog) TODO: Fix
+  ADD_TEST(dummy_test)
 END_TESTS
 
 
