@@ -1,21 +1,21 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -424,7 +424,7 @@ void t_strstr()
     type = strstr((const char *)string,(const char *)str);
     while (type++)
     {
-        int len = type - string;
+        int len = (int)(type - string);
         printMessage("\n Found '%s' at position '%d[%s]", str, len, type);
         type = strstr(type,str);
     }
@@ -440,7 +440,6 @@ void t_strstr()
 DECLARE_TEST(t_bug16920750)
 {
   SQLCHAR buff[255];
-  SQLLEN len;
 
   ok_sql(hstmt, "DROP SCHEMA IF EXISTS fk_test");
   ok_sql(hstmt, "CREATE SCHEMA fk_test");
@@ -497,7 +496,7 @@ DECLARE_TEST(t_bug16920750)
 
 
 BEGIN_TESTS
-  // ADD_TEST(my_foreign_keys)
+  ADD_TEST(my_foreign_keys)
   ADD_TEST(my_no_keys)
   ADD_TEST(t_bug16920750)
 END_TESTS
