@@ -1,21 +1,21 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1073,6 +1073,7 @@ DECLARE_TEST(t_bug29871)
   is_num(my_fetch_int(hstmt, 1), 1);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
   ok_sql(hstmt, "DROP TABLE t_bug29871");
   return OK;
 }
@@ -1171,6 +1172,7 @@ DECLARE_TEST(t_bug67702)
                                       "WHERE `id` = ? AND `vc` = ? AND "\
                                       "`yesno` = ?", SQL_NTS));
   SQLFreeStmt(hstmt, SQL_CLOSE);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
 
   /* Now check the result of update the bit field should be set to 0 */
   ok_stmt(hstmt, SQLExecDirect(hstmt, "SELECT `yesno` FROM `bug67702`", SQL_NTS));
@@ -1227,6 +1229,7 @@ DECLARE_TEST(t_bug68243)
                                       "WHERE `id` = ? AND `yesno` = ?",
                                       SQL_NTS));
   SQLFreeStmt(hstmt1, SQL_CLOSE);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_RESET_PARAMS));
 
   /* Now check the result of update the bit field should be set to 0 */
   ok_stmt(hstmt, SQLExecDirect(hstmt, "SELECT `yesno` FROM `bug68243`", SQL_NTS));
@@ -1279,6 +1282,7 @@ DECLARE_TEST(t_bug31667091)
                                   SQL_INTEGER, 0, 0, &id, 0, NULL));
   ok_stmt(hstmt, SQLExecute(hstmt));
   is_num(my_print_non_format_result(hstmt), 3);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
   ok_sql(hstmt,"DROP TABLE bug31667091");
   return OK;
 }
@@ -1302,7 +1306,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug67702)
   ADD_TEST(t_bug68243)
   ADD_TEST(t_bug67920)
-//  ADD_TODO(t_bug31667091)
+  ADD_TODO(t_bug31667091)
 END_TESTS
 
 
