@@ -1,30 +1,30 @@
-/*
-  Copyright 2003-2007 MySQL AB
-
-  The MySQL Connector/ODBC is licensed under the terms of the
-  GPL, like most MySQL Connectors. There are special exceptions
-  to the terms and conditions of the GPL as it is applied to
-  this software, see the FLOSS License Exception available on
-  mysql.com.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License as
-  published by the Free Software Foundation.
-
-  There are special exceptions to the terms and conditions of the GPL
-  as it is applied to this software. View the full text of the exception
-  in file LICENSE.exceptions in the top-level directory of this software
-  distribution.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+// Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// https://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
@@ -136,10 +136,9 @@ DECLARE_TEST(t_array_relative_10)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_array_relative_10");
 
-    rc = SQLExecDirect(hstmt,"create table t_array_relative_10(id int,name char(20))",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "create table t_array_relative_10(id int,name char(20))");
 
-    rc = SQLPrepare(hstmt,"insert into t_array_relative_10 values(?,?)",SQL_NTS);
+    rc = SQLPrepare(hstmt,(SQLCHAR *)"insert into t_array_relative_10 values(?,?)",SQL_NTS);
     mystmt(hstmt,rc);
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
@@ -171,8 +170,7 @@ DECLARE_TEST(t_array_relative_10)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_array_relative_10",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select * from t_array_relative_10");
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
@@ -282,10 +280,9 @@ DECLARE_TEST(t_relative_1)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_relative_1");
 
-    rc = SQLExecDirect(hstmt,"create table t_relative_1(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_relative_1(id int)");
 
-    rc = SQLPrepare(hstmt,"insert into t_relative_1 values(?)",SQL_NTS);
+    rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_relative_1 values(?)",SQL_NTS);
     mystmt(hstmt,rc);
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
@@ -311,8 +308,7 @@ DECLARE_TEST(t_relative_1)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_relative_1",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select * from t_relative_1");
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,&i,0,NULL);
     mystmt(hstmt,rc);
@@ -494,10 +490,9 @@ DECLARE_TEST(t_array_relative_2)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_array_relative_2");
 
-    rc = SQLExecDirect(hstmt,"create table t_array_relative_2(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt,"create table t_array_relative_2(id int)");
 
-    rc = SQLPrepare(hstmt,"insert into t_array_relative_2 values(?)",SQL_NTS);
+    rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_array_relative_2 values(?)",SQL_NTS);
     mystmt(hstmt,rc);
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
@@ -523,8 +518,7 @@ DECLARE_TEST(t_array_relative_2)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_array_relative_2",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select * from t_array_relative_2");
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
     mystmt(hstmt,rc);
@@ -730,7 +724,7 @@ DECLARE_TEST(t_array_relative_2)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_array_relative_2",SQL_NTS);
+    ok_sql(hstmt, "select * from t_array_relative_2");
     mystmt(hstmt,rc);
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
@@ -807,10 +801,9 @@ DECLARE_TEST(t_absolute_1)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_absolute_1");
 
-    rc = SQLExecDirect(hstmt,"create table t_absolute_1(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_absolute_1(id int)");
 
-    rc = SQLPrepare(hstmt,"insert into t_absolute_1 values(?)",SQL_NTS);
+    rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_absolute_1 values(?)",SQL_NTS);
     mystmt(hstmt,rc);
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
@@ -836,8 +829,7 @@ DECLARE_TEST(t_absolute_1)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_absolute_1",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select * from t_absolute_1");
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,&i,0,NULL);
     mystmt(hstmt,rc);
@@ -994,10 +986,9 @@ DECLARE_TEST(t_absolute_2)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_absolute_2");
 
-    rc = SQLExecDirect(hstmt,"create table t_absolute_2(id int)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_absolute_2(id int)");
 
-    rc = SQLPrepare(hstmt,"insert into t_absolute_2 values(?)",SQL_NTS);
+    rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_absolute_2 values(?)",SQL_NTS);
     mystmt(hstmt,rc);
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT, SQL_C_ULONG,
@@ -1023,7 +1014,7 @@ DECLARE_TEST(t_absolute_2)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_absolute_2",SQL_NTS);
+    ok_sql(hstmt, "select * from t_absolute_2");
     mystmt(hstmt,rc);
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
@@ -1190,7 +1181,7 @@ DECLARE_TEST(t_absolute_2)
     rc = SQLSetStmtAttr(hstmt,SQL_ATTR_ROWS_FETCHED_PTR,&nrows,0);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt,"select * from t_absolute_2",SQL_NTS);
+    ok_sql(hstmt, "select * from t_absolute_2");
     mystmt(hstmt,rc);
 
     rc = SQLBindCol(hstmt,1,SQL_C_LONG,iarray,0,NULL);
