@@ -1,21 +1,21 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
 // published by the Free Software Foundation.
 //
-// This program is also distributed with certain software (including
-// but not limited to OpenSSL) that is licensed under separate terms,
-// as designated in a particular file or component or in included license
-// documentation. The authors of MySQL hereby grant you an
-// additional permission to link the program and your derivative works
-// with the separately licensed software that they have included with
-// MySQL.
+// This program is designed to work with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms, as
+// designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an additional
+// permission to link the program and your derivative works with the
+// separately licensed software that they have either included with
+// the program or referenced in the documentation.
 //
 // Without limiting anything contained in the foregoing, this file,
-// which is part of <MySQL Product>, is also subject to the
+// which is part of Connector/ODBC, is also subject to the
 // Universal FOSS Exception, version 1.0, a copy of which can be found at
-// http://oss.oracle.com/licenses/universal-foss-exception.
+// https://oss.oracle.com/licenses/universal-foss-exception.
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -285,8 +285,8 @@ DECLARE_TEST(sqldriverconnect)
   }
 
   ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(conn_in, wcslen(conn_in)),
-                                  wcslen(conn_in), conn_out,
-                                  sizeof(conn_out)/sizeof(SQLWCHAR),
+                                  (SQLSMALLINT)wcslen(conn_in), conn_out,
+                                  (SQLSMALLINT)(sizeof(conn_out)/sizeof(SQLWCHAR)),
                                   &conn_out_len, SQL_DRIVER_NOPROMPT));
 
   ok_con(hdbc, SQLAllocStmt(hdbc1, &hstmt1));
@@ -1173,7 +1173,7 @@ DECLARE_TEST(t_bug28168)
   /* Connect using UTF8 as transport to avoid server bug with user names */
   ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(work_conn_in,
                                   wcslen(work_conn_in)),
-                                  wcslen(work_conn_in), NULL, 0,
+                                  (SQLSMALLINT)wcslen(work_conn_in), NULL, 0,
                                   0, SQL_DRIVER_NOPROMPT));
 
   ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
@@ -1310,8 +1310,8 @@ DECLARE_TEST(t_bug14363601)
   wcscat(conn_in, L";CHARSET=utf16");
 
   ok_con(hdbc1, SQLDriverConnectW(hdbc1, NULL, WL(conn_in, wcslen(conn_in)),
-                                  wcslen(conn_in), conn_out,
-                                  sizeof(conn_out)/sizeof(SQLWCHAR),
+                                  (SQLSMALLINT)wcslen(conn_in), conn_out,
+                                  (SQLSMALLINT)(sizeof(conn_out)/sizeof(SQLWCHAR)),
                                   &conn_out_len, SQL_DRIVER_NOPROMPT));
 
   ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
